;****************************************************************************
;                         CALCULATION MODULE
;
;         (uses the greater-precision SWRx100 in the R computation)
;
;****************************************************************************

;****************************************************************************
; Do_Calcs -- Do all the calculations, as specified by the Mode, and disiplay accordingly

Do_Calcs:
        jsr     Calc_SWR
        jsr     Calc_Z
        jsr     Calc_R
        jsr     Calc_X

        lda     Mode_nr
        cmp     #Inductance_Mode        ;Inductance Mode?
        bne     dc_1                    ;no
        jsr     Calc_L                  ;yes, calc Inductance and display results on 2nd line
        bra     dc_exit

dc_1:   cmp     #Capacitance_Mode       ;Cap mode?
        bne     dc_2                    ;no
        jsr     Calc_C                  ;yes, calc Inductance and display results on 2nd line
        bra     dc_exit
        lda     Mode_nr

dc_2:   cmp     #VFO_Mode               ;VFO mode?
        bne     dc_exit                 ;no, just exit
        lda     #$C6                    ;yes, display "VFO" on second line
        jsr     Cmnd2LCD
        ldhx    #VFO_msg2
        jsr     Disp_Msg

dc_exit:
        ldhx    cursor_index          ;put cursor back up on digit of freq being modified
        lda     digit_posn,X          ;get LCD digit address
        jsr     cmnd2LCD              ;and put cursor back up there on Freq line
        rts

;****************************************************************************
; Calc_SWR

Calc_SWR:
             ;-------------------------------------------
             ;Read Vf and multiply by 100 (for resolution)
             ;-------------------------------------------
        lda     #CHAN_Vf
        jsr     read_ADC                ;read Vf
        sta     Vf
        jsr     dpr_A_nocr              ;debug print Vf_raw
        sta     intacc1+3
        clr     intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        lda     Vf_offset               ;subract offset
        jsr     dpr_A_nocr              ;debug print Vf offset
        sta     intacc2+3
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     usub32                  ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        lda     intacc1+3
        jsr     dpr_A                   ;debug print Vf_offset_adjusted
        mov     intacc1+3,intacc1+1
        mov     intacc1+2,intacc1+0
        mov     #$64,intacc2+1
        clr     intacc2+0
        jsr     umult16                 ;Vfx100
        jsr     dpr_acc1                ;Debug print Vfx100
        ldhx    #Vfx100
        jsr     store_acc1              ;store intacc1(0:3) at Vfx100

             ;---------------------------------------------------------------
             ;Read Vr and multiply by proper correction factor (for resolution and correction)
             ;---------------------------------------------------------------
calc01: lda     #CHAN_Vr
        jsr     read_ADC                ;read Vr
        sta     Vr
        jsr     dpr_A_nocr              ;debug print Vr raw
        sta     intacc1+3
        clr     intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        lda     Vr_offset               ;subract offset
        jsr     dpr_A_nocr              ;debug print Vf offset
        sta     intacc2+3
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     usub32                  ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        lda     intacc1+0               ;see if neg result (baaaaaad)
        bpl     calc01a                 ;positive result, so all okay ... continue
        clr     intacc1+0               ;neg result, or offset > reading (possible only with low SWR)
        clr     intacc1+1               ;so just make result 0
        clr     intacc1+2
        clr     intacc1+3
calc01a:
        lda     intacc1+3
        jsr     dpr_A_nocr              ;debug print Vr_offset_adj
        mov     intacc1+3,intacc1+1     ;move offset-corrected Vr into intacc1(0:1)
        mov     intacc1+2,intacc1+0
        lda     Vr_corrfact
        jsr     dpr_A                   ;debug print Vf_correction_factor
        sta     intacc2+1
        clr     intacc2+0
        jsr     umult16                 ;correct Vr: Vr x Vr_corrfact
        jsr     dpr_acc1                ;Debug print Vr_acc
        ldhx    #Vr_acc
        jsr     store_acc1              ;store corrected Vr reading at Vr_acc

        jsr     MakeSWRtone             ;generate tone based on SWR

             ;--------------------------------
             ;PERFORM: SWR = (Vf+Vr) / (Vf-Vr)
             ;--------------------------------

             ;first: Sum = (Vf+Vr)
calc05: ldhx    #Vfx100
        jsr     load_acc1
        ldhx    #Vr_acc
        jsr     load_acc2
        jsr     uadd32                ;intacc1(0:3) + intacc2(0:3) = intacc1(0:3)
        mov     #$64,intacc2+3        ;multiply Sum x 100 to ultimately end up with SWRx100
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     umult32               ;intacc1(0:3) x intacc2(0:3) = intacc1(0:3)...intacc2(0:3)
        mov     intacc2+0,intacc1+0   ;transfer LS word into intacc1(0:3)
        mov     intacc2+1,intacc1+1
        mov     intacc2+2,intacc1+2
        mov     intacc2+3,intacc1+3
        jsr     dpr_acc1              ;Debug print (Vf+Vr)*100
        ldhx    #temp1_acc
        jsr     store_acc1            ;temp save (Vf+Vr)*100

             ;Diff = Vf-Vr
        ldhx    #Vfx100
        jsr     load_acc1             ;get Vfx100 into intacc1(0:3)
        ldhx    #Vr_acc
        jsr     load_acc2             ;get corrected Vr reading into intacc2
        jsr     usub32                ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        jsr     dpr_acc1              ;Debug print (Vf-Vr)
        ldhx    #temp2_acc
        jsr     store_acc1            ;temp save (Vf-Vr)
        lda     intacc1+0
        cmp     #$FF                  ;if diff is pos,
        bne     calc6                 ;we're good to go for computing SWR
        clr     intacc1+0             ;otherwise, clamp denominator to 1
        clr     intacc1+1             ;so SWR computation doesn't blow up
        clr     intacc1+2
        mov     #2,intacc1+3
        ldhx    #temp2_acc
        jsr     store_acc1            ;temp save clamped diff value

             ;Sum / Diff
calc6:  ldhx    #temp1_acc
        jsr     load_acc1             ;get the SUMx100 into intacc1(0:3)
        lda     temp2_acc+3
        sta     intacc2+1             ;get the DIFF into intacc2(0:1)
        lda     temp2_acc+2
        sta     intacc2+0
        jsr     udvd32                ;intacc1(0:3) / intacc2(0:1) = intacc1(0:3)
        ldhx    #SWRx100              ;save value of SWRx100
        jsr     store_acc1
        jsr     dpr_acc1              ;Debug print SWRx100

             ;---------------------------------------------------------------
             ;Read Va and multiply by proper correction factor (for resolution and correction)
             ;---------------------------------------------------------------
        lda     #CHAN_Va
        jsr     read_ADC                ;read Va
        sta     Va
        jsr     dpr_A_nocr              ;debug print Va_raw
        sta     intacc1+3
        clr     intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        lda     Va_offset               ;subtract offset
        jsr     dpr_A_nocr              ;debug print Va_offset
        sta     intacc2+3
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     usub32                  ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        lda     intacc1+3
        jsr     dpr_A_nocr              ;debug print Va_offset_adj
        mov     intacc1+3,intacc1+1     ;move offset-corrected Va into intacc1(0:1)
        mov     intacc1+2,intacc1+0

             ;Va_corrected = (Va_offset_adj x Va_corrfact_whole#)
             ;               + (Va_offset_adj x Va_corrfact_remainder / 256)

        ldhx    #temp2_acc
        jsr     store_acc1              ;temp save Va_offset_adjusted
        lda     Va_corrfact
        jsr     dpr_A_nocr              ;debug print Va_correction_factor
        lda     Va_corrfact+1
        jsr     dpr_A                   ;debug print Va_corrfact remainder
        lda     Va_corrfact
        sta     intacc2+1
        clr     intacc2+0
        jsr     umult16                 ;correct Va: Va x Va_corrfact+0
        ldhx    #temp1_acc
        jsr     store_acc1              ;temporarily save whole number part of Va_corrected

        lda     Va_corrfact+1           ;load up remainder into intacc1(0:1)
        sta     intacc1+1
        clr     intacc1+0
        lda     temp2_acc+2             ;load up LS two bytes of Va_offset_adjusted
        sta     intacc2+0
        lda     temp2_acc+3
        sta     intacc2+1
        jsr     umult16                 ;multiply: Va_offset_adjusted x remainder

        mov     intacc1+2,intacc1+3     ;shift intacc1(0:3) right by 8 bits,
        mov     intacc1+1,intacc1+2     ;  thus dividing result of (Va_offset_adjusted x remainder) by 256
        mov     intacc1+0,intacc1+1     ;  to yield significant digits to add to Va_corrected
        clr     intacc1+0

        ldhx    #temp1_acc
        jsr     load_acc2               ;get back whole number part of Va_corrected into intacc2
        jsr     uadd32                  ;Vz_corrected = whole part + remainder part
        jsr     dpr_acc1                ;Debug print Va_acc
        ldhx    #Va_acc
        jsr     store_acc1              ;store corrected Va reading at Va_acc


             ;--------------------------------------------------
             ;Read Vz and multiply by proper Vz correction value
             ;from Cal_Struct (for resolution and correction)
             ;--------------------------------------------------
makeVz: lda     #CHAN_Vz
        jsr     read_ADC                ;read Vz
        sta     Vz
        jsr     dpr_A_nocr              ;debug print Vz_raw
        sta     intacc1+3
        clr     intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        lda     Vz_offset               ;subtract offset
        jsr     dpr_A_nocr              ;debug print Vz_offset
        sta     intacc2+3
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     usub32                  ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        lda     intacc1+3
        jsr     dpr_A_nocr              ;debug print Vz_offset_adjusted
        mov     intacc1+3,intacc1+1     ;move offset-corrected Vz into intacc1(0:1)
        mov     intacc1+2,intacc1+0

             ;Vz_corrected = (Vz_offset_adj x Vz_corrfact_whole#)
             ;               + (Vz_offset_adj x Vz_corrfact_remainder / 256)

        ldhx    #temp2_acc
        jsr     store_acc1              ;temp save Vz_offset_adjusted
        lda     Vz_corrfact
        jsr     dpr_A_nocr              ;debug print Vz_correction_factor
        lda     Vz_corrfact+1
        jsr     dpr_A                   ;debug print Vz_corrfact remainder
        lda     Vz_corrfact
        sta     intacc2+1
        clr     intacc2+0
        jsr     umult16                 ;correct Vz: Vz x Vz_corrfact
        ldhx    #temp1_acc
        jsr     store_acc1              ;temporarily save whole number part of Vz_corrected

        lda     Vz_corrfact+1           ;load up remainder into intacc1(0:1)
        sta     intacc1+1
        clr     intacc1+0
        lda     temp2_acc+2             ;load up LS two bytes of Vr_offset_adjusted
        sta     intacc2+0
        lda     temp2_acc+3
        sta     intacc2+1
        jsr     umult16                 ;multiply: Vz_offset_adjusted x remainder

        mov     intacc1+2,intacc1+3     ;shift intacc1(0:3) right by 8 bits,
        mov     intacc1+1,intacc1+2     ;  thus dividing result of (Vr_offset_adjusted x remainder) by 256
        mov     intacc1+0,intacc1+1     ;  to yield significant digits to add to Vz_corrected
        clr     intacc1+0

        ldhx    #temp1_acc
        jsr     load_acc2               ;get back whole number part of Vz_corrected into intacc2
        jsr     uadd32                  ;Vz_corrected = whole part + remainder part
        jsr     dpr_acc1                ;Debug print Vz_acc
        ldhx    #Vz_acc
        jsr     store_acc1              ;store corrected Vr reading at Vz_acc

             ;check for SWR beyond limit (max: 00_00_03_DE = 990. = SWR of 9.90)
calc6d: tst     SWRx100+0
        bne     clamp                 ;beyond limit if there is msd
        tst     SWRx100+1
        bne     clamp                 ;same for 2nd msd
        pshhx
        ldhx    SWRx100+2             ;next check two LSDs [SWRx100(2:3)]
        cphx    #$03DE
        pulhx
        bcc     clamp                 ;beyond limit, clamp SWR at 9.9

             ;Convert intacc1(2:3) result to BCD
             ;example: intacc1: 00_00_00_A8 = "168" = SWR of 1.68

        ldhx    #SWRx100              ;get SWRx100(0:3) into intacc1(0:3)
        jsr     Load_acc1
        jsr     Bin2Bcd               ;convert intacc1(2:3) to BCD in bcd100_bcd10_bcd1

             ;save WW & DD
        mov     bcd100,WW             ;save digit as SWR whole number
        mov     bcd10,DD              ;save digit as SWR decimal
        lda     bcd1
        cmp     #5                    ;round up decimal, if necessary
        beq     calc4                 ;round up if digit =5
        bcc     calc4                 ;round up if digit >5
        bra     Disp_SWR              ;not >=5, just display SWR and exit
calc4:  inc     DD                    ;otherwise, we need to round up
        lda     #10
        cmp     DD                    ;rollover?
        bne     Disp_SWR              ;nope, just continue
        clr     DD                    ;yes, do the rollover
        inc     WW                    ;round up WW
        lda     #10
        cmp     WW                    ;was WW at-9-going-to-10?
        bne     Disp_SWR              ;nope, leave rollover alone
        bra     clamp                 ;yup, clamp SWR

             ; Disp_SWR - display the SWR as W.D
Disp_SWR:
        tst     DDS_active            ;bypass display of SWR if DDS not active (i.e., during cal)
        bne     dswr1
        rts
dswr1:  lda     No_Disp_Flag          ;don't display if the No_Disp_Flag is set (during X_Sign "freq diddle")
        beq     dswr1a
        rts
dswr1a: lda     SWR_Disp_Flag
        bne     dswr2                 ;display SWR only if flag set
        rts

             ;Display SWR number W.D
dswr2:  lda     #$C0		      ;point to digit position on 2nd line
	jsr	Cmnd2LCD
        lda     WW                    ;get whole number result of divide
        jsr     _nib_to_asc           ;convert the lower nibble to ascii
        jsr     data2LCD              ;and display it as the units digit of SWR whole number
        lda     #'.'                  ;display decimal point
        jsr     data2LCD
        lda     DD                    ;get decimal remainder
        jsr     _nib_to_asc           ;convert the lower nibble to ascii
        jsr     data2LCD              ;and display it as the tenths digit of SWR remainder
        ldhx    cursor_index          ;put cursor back up on digit of freq being modified
        lda     digit_posn,X          ;get LCD digit address
        jsr     cmnd2LCD              ;and put cursor back up there on Freq line
        rts

             ;clamp SWR  ... Display '>10'
clamp:  ldhx    #$03DE                ;put SWRx100(2:3) to max SWR value 9.9 (990 = 03DE)
        sthx    SWRx100+2
        tst     DDS_active
        beq     clamp2                ;bypass display of SWR if flag clear
        lda     SWR_Disp_Flag
        beq     clamp2                ;bypass display of SWR if flag clear

        lda     #$C0		      ;point to digit position on 2nd line
	jsr	Cmnd2LCD
        ldhx    #SWR_clamp
        jsr     Disp_Msg
        ldhx    cursor_index          ;put cursor back up on digit of freq being modified
        lda     digit_posn,X          ;get LCD digit address
        jsr     cmnd2LCD              ;and put cursor back up there on Freq line

        lda	#9                    ; max out the integer
        sta	bcd100
        sta	bcd10
        sta	bcd1

clamp2: rts

SWR_clamp: fcb   '>10',0

;****************************************************************************
; Calc_Z
;
;           Z = 50 (Vz_acc/Va_acc)

Calc_Z: lda     Vz_acc+3             ;get Vz_acc into intacc1(0:1)
        sta     intacc1+1
        lda     Vz_acc+2
        sta     intacc1+0
        mov     #50,intacc2+1         ;get 50 into intacc2(0:1)
        clr     intacc2+0
        jsr     umult16               ;intacc1(0:1) x intacc2(0:1) = intacc1(0:3)
        lda     Va_acc+3             ;get corrected Va value (about x100) into intacc2(0:1)
        sta     intacc2+1
        lda     Va_acc+2
        sta     intacc2+0
        jsr     udvd32                ;intacc1(0:3) / intacc2(0:1) = intacc1(0:3)
        jsr     dpr_acc1              ;Debug print Z

            ;Test for Z > 600   (max of 600 = 00_00_02_58)
        tst     intacc1+0
        bne     clamp_Z               ;beyond limit if there is msd
        tst     intacc1+1
        bne     clamp_Z               ;same for 2nd msd
        pshhx
        ldhx    intacc1+2             ;next check two LSDs [SWRx100(2:3)]
        cphx    #$0258
        pulhx
        bcc     clamp_Z               ;beyond limit
        clra
        sta     Z_maxed               ;else, in range, so clear flag and exit successfully!
        rts
            ;Set flag if Z max has been exceeded.  (To later bypass R & X calculations.)
clamp_Z:
        lda     #1
        sta     Z_maxed               ;set flag if Z max has been exceeded
        rts

;****************************************************************************
; Calc_R
;               (2500 + Z^2) * SWR
;           R = ------------------
;                50 * (SWR^2 + 1)
;      or ...
;                2*(2500+Z^2)*(SWR*100)
;           R =  ----------------------
;                  (SWR*100)^2+10000
;
;           enter with Z in intacc1(0:3)

Calc_R:
             ;Work on NUMERATOR: (2500 + Z^2) ==> R_temp(0:3)
             ;Perform: Z^2
cr0:    mov     intacc1+3,intacc2+1   ;move lower 2 bytes of Z into intacc2(0:1)
        mov     intacc1+2,intacc2+0
        mov     intacc2+1,intacc1+1   ;move same 2 bytes into intacc1(0:1)
        mov     intacc2+0,intacc1+0
        jsr     umult16               ;intacc1(0:1) x intacc2(0:1) = intacc1(0:3)
        mov     intacc1+0,z_squared+0
        mov     intacc1+1,z_squared+1
        mov     intacc1+2,z_squared+2
        mov     intacc1+3,z_squared+3
        jsr     dpr_acc1              ;Debug print Z^2

             ;add 2500
        clr     intacc2+0             ;2500. = $09C4
        clr     intacc2+1
        mov     #$09,intacc2+2
        mov     #$C4,intacc2+3
        jsr     uadd32                ;intacc1(0:3) + intacc2(0:3) = intacc1(0:3)

             ;multiply by 2
        clr     intacc2+0
        clr     intacc2+1
        clr     intacc2+2
        mov     #2,intacc2+3
        jsr     umult32               ;intacc1(0:3) x intacc2(0:3) = intacc1(0:3)...intacc2(0:3)

             ;multiply by SWRx100 and save numerator
        ldhx    #SWRx100
        jsr     load_acc1
        jsr     umult32               ;intacc1(0:3) x intacc2(0:3) = intacc1(0:3)...intacc2(0:3)
cr1:    mov     intacc2+0,r_temp+0
        mov     intacc2+1,r_temp+1
        mov     intacc2+2,r_temp+2
        mov     intacc2+3,r_temp+3
        jsr     dpr_acc2              ;Debug print NUMERATOR  (r_temp)

             ;Work on DENOMINATOR:  (SWR*100)^2 + 10,000 ==> SWR_temp(0:3)
             ;perform (SWRx100)^2
        mov     SWRx100+0,intacc1+0   ;load intacc1(0:3) with SWRx100
        mov     SWRx100+1,intacc1+1
        mov     SWRx100+2,intacc1+2
        mov     SWRx100+3,intacc1+3
        mov     SWRx100+0,intacc2+0   ;load intacc2(0:3) with SWRx100
        mov     SWRx100+1,intacc2+1
        mov     SWRx100+2,intacc2+2
        mov     SWRx100+3,intacc2+3
        jsr     umult32               ;intacc1(0:3) x intacc2(0:3) = intacc1(0:3)...intacc2(0:3)

            ;add 10,000
        clr     intacc1+0
        clr     intacc1+1
        mov     #$27,intacc1+2
        mov     #$10,intacc1+3
        jsr     uadd32                ;intacc1(0:3) = intacc1(0:3) + intacc2(0:3)
        ldhx    #SWR_temp
        jsr     store_acc1
        jsr     dpr_acc1              ;Debug print DENOMINATOR   (SWR_temp)

             ;-------------------------
             ;Divide numerator by denominator
             ;-------------------------

             ;If SWR_temp too big (SWR_temp(0:1) <>0), then must scale division
        ldhx    SWR_temp              ;get SWR_temp(0:1) into HX
        beq     cr3a                  ;both upper bytes are zero, so no need to scale accumulators
        lda     SWR_temp+0
        bne     cr2                   ;upper byte is non-zero, so must scale by 65,535

             ;SWR_temp+0 is 00 but SWR_temp+1 is non-zero, so must divide numerator and denominator
             ;by 256 to fit accumulators (so division by 2-byte denominator is okay)
        mov     r_temp+2,intacc1+3   ;get LSBs of numerator (r_temp) into intacc1(0:3)
        mov     r_temp+1,intacc1+2
        mov     r_temp+0,intacc1+1
        clr     intacc1+0
        mov     SWR_temp+2,intacc2+1
        mov     SWR_temp+1,intacc2+0  ;get LSBs of demominator into intacc2(0:1)
        bra     cr3b                  ;continue with division

             ;SWR_temp+0 is non-zero, so must divide numerator and denominator
             ;by 65,535 to fit accumulators (so division by 2-byte denominator is okay)
cr2:    mov     r_temp+1,intacc1+3   ;get two LS bytes of numerator (r_temp) into intacc1(0:3)
        mov     r_temp+0,intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        mov     SWR_temp+1,intacc2+1
        mov     SWR_temp+0,intacc2+0  ;get LS bytes of demominator into intacc2(0:1)
        bra     cr3b                  ;continue with division

             ;No need to scale accumulators before division
cr3a:   mov     r_temp+3,intacc1+3   ;get intacc2(0:3) [SWR*R_temp] into intacc1(0:3) [numerator]
        mov     r_temp+2,intacc1+2
        mov     r_temp+1,intacc1+1
        mov     r_temp+0,intacc1+0
        mov     SWR_temp+3,intacc2+1
        mov     SWR_temp+2,intacc2+0  ;get LSB of SWR_temp (0:3) into intacc2(0:1) [denominator]

             ;Continue with the division
cr3b:   jsr     udvd32                ;intacc1(0:3) / intacc2(0:1) = intacc1(0:3)
        mov     intacc1+0,R+0         ;save R at R(0:3)
        mov     intacc1+1,R+1
        mov     intacc1+2,R+2
        mov     intacc1+3,R+3
        jsr     dpr_acc1              ;Debug print R

             ;If Z calculation was > 600, then don't display R
        lda     Z_maxed
        beq     cr3c                  ;not maxed out, so just continue to display R

        clra                          ;Z is maxed, R is meaningless, so put R=0 (for Link908 display)
        sta     R
        sta     R+1
        sta     R+2
        sta     R+3

        lda     R_Disp_Flag
        beq     cr3e                  ;don't display "Z>600" if flag not set

        lda     #$C3
        jsr     Cmnd2LCD
        ldhx    #Z_err                ;else display 'Z>600'
        jsr     Disp_Msg
cr3e:   ldhx    cursor_index          ;get LCD digit address
        lda     digit_posn,X
        jsr     Cmnd2LCD              ;put cursor back up on the Frequency line
        rts

             ;if R > 2048, then display max of 2048
cr3c:   tst     intacc1+0
        bne     R_high
        tst     intacc1+1
        bne     R_high
        lda     #$08
        cmp     intacc1+2             ;is intacc1 > 0800?  (0800 = 2048.)
        bcc     R_ok                  ;no, R is less than 0800 and thus in range

             ;R too high so just display '2048'
R_high: clr     R+0                   ;clamp R(0:3) = 00 00 08 00 = 2048 ohms
        clr     R+1
        mov     #$08,R+2
        clr     R+3
        clr     intacc1+0             ;also clamp intacc1(0:3) = 00 00 08 00 = 2048 ohms
        clr     intacc1+1
        mov     #$08,intacc1+2
        clr     intacc1+3

             ;-------------------------
             ; Display Resistance Value
             ;-------------------------

R_ok:   lda     No_Disp_Flag          ;don't display if the No_Disp_Flag is set
        beq     R_ok1
        rts
R_ok1:  lda     R_Disp_Flag
        bne     cr9                   ;display R only if flag set
        rts

             ;convert R to BCD and display it

cr9:    jsr     Bin2BCD               ;convert intacc1(2:3) to BCD in bcd100_bcd10_bcd1

             ;first erase 'Z>600' (if there) and re-display ' +j  '
        lda     #$C7
        jsr     Cmnd2LCD
        ldhx    #ClrZ600
        jsr     Disp_Msg              ;overwrite 5 bytes of 'Z>600'
        tst     DDS_active
        beq     cr4b                  ;bypass display of '+j' if flag clear
        lda     #$C3
        jsr     Cmnd2LCD              ;point to 2nd line
        ldhx    #J_msg                ;display "          +j    " on line 2
        jsr     disp_msg
        lda     #$CA
        jsr     Cmnd2LCD              ;point to sign position on line 2
        lda     sign                  ;get the sign, as deterimed in module "X_sign.asm", computed in ISR
        jsr     Data2LCD              ;and display the sign

cr4b:   lda     #$C5                  ;point to position 6 on line 2
        jsr     Cmnd2LCD
        lda     bcd1000               ;display 1000s digit
        tsta
        bne     cr4
        lda     #' '                  ;don't display leading zero
        bra     cr4a
cr4:    jsr     _nib_to_asc
cr4a:   jsr     data2LCD

             ;display 100s digit
        lda     #$C6
        jsr     Cmnd2LCD
        lda     bcd100
        tsta
        bne     cr5                   ;100s digit is non-zero, so display it
        lda     bcd1000               ;but if zero, check preceeding digit
        tsta                          ;also zero?
        bne     cr5                   ;no, display it
        lda     #' '                  ;yes, display blank instead of zero
        bra     cr5a
cr5:    lda     bcd100                ;display 100s digit
        jsr     _nib_to_asc
cr5a:   jsr     data2LCD

            ;display 10s digit
cr6:    lda     #$C7
        jsr     Cmnd2LCD
        lda     bcd10
        tsta
        bne     cr7                   ;10s digit is non-zero, so display it
        lda     bcd100                ;but if zero, check preceeding 100s digit for zero
        tsta                          ;also zero?
        bne     cr7                   ;no, need to display this zero in 10s position
        lda     bcd1000               ;yes, so see if 1000s digit also zero
        bne     cr7                   ;no, so display the 10s digit
        lda     #' '                  ;else, just display space instead of zero
        bra     cr7a
cr7:    lda     bcd10                 ;display 10s digit
        jsr     _nib_to_asc
cr7a:   jsr     data2LCD

             ;display 1s digit, even if it's zero
cr8:    lda     #$C8
        jsr     Cmnd2LCD
        lda     bcd1                  ;display 1s digit
        jsr     _nib_to_asc
        jsr     data2LCD
        ldhx    cursor_index          ;put cursor back up on digit of freq being modified
        lda     digit_posn,X          ;get LCD digit address
        jsr     cmnd2LCD              ;and put cursor back up there on Freq line
        rts

Z_err:   fcb '    Z>600    ',0
J_msg:   fcb '       +j     ',0
ClrZ600: fcb '     ',0

;****************************************************************************
; Calc_X
;           X = SQRT ( Z_squared - R_squared )

Calc_X:
            ;Perform: R_squared
cx1:    mov     R+2,intacc2+0
        mov     R+3,intacc2+1
        mov     R+2,intacc1+0
        mov     R+3,intacc1+1
        jsr     umult16               ;intacc1(0:1) x intacc2(0:1) = intacc1(0:3)
        jsr     dpr_acc1              ;Debug print R^2

            ;Perform: Z_sq(0:3) - intacc1(0:3) ==> intacc1(0:3)
        lda     Z_squared+3
        sub     intacc1+3             ; [LSB] Z-R
        sta     intacc1+3
        lda     Z_squared+2
        sbc     intacc1+2
        sta     intacc1+2
        lda     Z_squared+1
        sbc     intacc1+1
        sta     intacc1+1
        lda     Z_squared+0
        sbc     intacc1+0
        sta     intacc1+0
        jsr     dpr_acc1              ;Debug print Z^2 - R^2

             ;If Z calculation was > 600, then don't calculate or display X
        lda     Z_maxed
        beq     cx2                   ;not maxed out, so just continue to display X
                                      ;  ('Z>600' was displayed from R_calc routine)
        ldhx    #100                  ;but perform calc X anyway to keep DEBUG data record intact
        jsr     sq_root
        ldhx    #0                    ;if Z>600 then X is meaningless, so clear out Reactance (for Link908 plot)
        sthx    Reactance
        jsr     dpr_acc1              ;Debug print X = SQRT(Z^2-R^2)
        rts

            ;Test for negative Z-R result

cx2:    brclr   7,intacc1+0,x_ok      ;MSB of difference is 0 if Z-R = positive = good

        lda     X_Disp_Flag
        beq     cx2a                  ;don't display reactance if flag not set

        lda     #$CC                  ;else just display 0 and exit
        jsr     Cmnd2LCD
        lda     #' '
        jsr     Data2LCD
        lda     #' '
        jsr     Data2LCD
        lda     #' '
        jsr     Data2LCD
        lda     #'0'
        jsr     Data2LCD
        clr     intacc1+0
        clr     intacc1+1
        clr     intacc1+2
        clr     intacc1+3
        jsr     dpr_acc1              ;Debug print SQRT error placeholder for DEBUG data record
        jsr     dpr_crlf
        ldhx    #0
        sthx    Reactance

cx2a:   ldhx    cursor_index          ;put cursor back up on digit of freq being modified
        lda     digit_posn,X          ;get LCD digit address
        jsr     cmnd2LCD              ;and put cursor back up on Freq line
        rts

            ;Perform: SQRT

x_ok:   ldhx    #100
        jsr     sq_root
        ldhx    intacc1+2             ;get least significant two bytes of X
        sthx    Reactance             ;and store them
        jsr     dpr_acc1              ;Debug print X = SQRT(Z^2-R^2)
        jsr     dpr_crlf

             ;------------------------
             ; Display Reactance Value
             ;------------------------

        lda     No_Disp_Flag          ;don't display if the No_Disp_Flag is set
        beq     x_ok1
        rts
x_ok1:  lda     X_Disp_Flag
        bne     cx9                   ;display X only if flag set
        rts

             ;convert X to BCD and display it

cx9:    jsr     Bin2BCD               ;convert intacc1(2:3) to BCD in bcd10000_bcd_1000_bcd100_bcd10_bcd1
        lda     bcd10                 ;shift BCD #'s right one position, dividing by 10
        sta     bcd1
        lda     bcd100
        sta     bcd10
        lda     bcd1000
        sta     bcd100
        lda     bcd10000
        sta     bcd1000

        lda     #$CC                  ;point to position 13 on line 2
        jsr     Cmnd2LCD
        lda     bcd1000               ;display 1000s digit
        tsta
        bne     cx4
        lda     #' '                  ;don't display leading zero
        bra     cx4a
cx4:    jsr     _nib_to_asc
cx4a:   jsr     data2LCD

             ;display 100s digit
        lda     #$CD
        jsr     Cmnd2LCD
        lda     bcd100
        tsta
        bne     cx5                   ;100s digit is non-zero, so display it
        lda     bcd1000               ;but if zero, check preceeding digit
        tsta                          ;also zero?
        bne     cx5                   ;no, display it
        lda     #' '                  ;yes, display blank instead of zero
        bra     cx5a
cx5:    lda     bcd100                ;display 100s digit
        jsr     _nib_to_asc
cx5a:   jsr     data2LCD

            ;display 10s digit
cx6:    lda     #$CE
        jsr     Cmnd2LCD
        lda     bcd10
        tsta
        bne     cx7                   ;10s digit is non-zero, so display it
        lda     bcd100                ;but if zero, check preceeding 100s digit for zero
        tsta                          ;also zero?
        bne     cx7                   ;no, need to display this zero in 10s position
        lda     bcd1000               ;yes, so see if 1000s digit also zero
        bne     cx7                   ;no, so display the 10s digit
        lda     #' '                  ;else, just display space instead of zero
        bra     cx7a
cx7:    lda     bcd10                 ;display 10s digit
        jsr     _nib_to_asc
cx7a:   jsr     data2LCD

             ;display 1s digit, even if it's zero
cx8:    lda     #$CF
        jsr     Cmnd2LCD
        lda     bcd1                  ;display 1s digit
        jsr     _nib_to_asc
        jsr     data2LCD
        ldhx    cursor_index          ;put cursor back up on digit of freq being modified
        lda     digit_posn,X          ;get LCD digit address
        jsr     cmnd2LCD              ;and put cursor back up there on Freq line
        rts

;*****************************************************************
; Calc_L - Calculate the inductance: L= ((X*10)*10000)/((63)*F)

Calc_L:
        lda     Z_maxed               ;check for Z>600
        beq     cl_05                 ;it's less than, so proceed
        lda     #$C6
        jsr     Cmnd2LCD
        ldhx    #Z_err2               ;display (Z>600)
        jsr     Disp_Msg
        jmp     cl_exit

cl_05:  ldhx    Reactance             ;check for X < 1.0
        cphx    #$000A
        bcc     cl_1                  ;it's > that, so proceed
        lda     #$C3
        jsr     Cmnd2LCD
        lda     #'L'                  ;else display 'L= ' (X=0)
        jsr     Data2LCD
        lda     #'='
        jsr     Data2LCD
        lda     #' '
        jsr     Data2LCD
        ldhx    #X_err2               ;display (X=0)
        jsr     Disp_Msg
        jmp     cl_exit

             ;calculate numerator: (X*10 x 100,000)
cl_1:   ldhx    Reactance             ;get X computation (it's really X*10)
        sthx    intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        clr     intacc2+0
        mov     #$01,intacc2+1        ;load 100,000 into intacc2(0:3)
        mov     #$86,intacc2+2
        mov     #$A0,intacc2+3
        jsr     umult32               ;numerator
        mov     intacc2+0,intacc1+0
        mov     intacc2+1,intacc1+1
        mov     intacc2+2,intacc1+2
        mov     intacc2+3,intacc1+3

             ;divide by 628
        mov     #$02,intacc2+0        ;0274 = 628.
        mov     #$74,intacc2+1
        jsr     udvd32

             ;divide by Freq
        jsr     GetFreq               ;gets LCD_Dat(0:4) into binary in HX (1 kHz resolution)
        cphx    #0                    ;check for F=0
        bne     cl_2                  ;F>0, so continue
        lda     #$C6                  ;F=0, so display (X=0)
        jsr     Cmnd2LCD
        ldhx    #X_err2               ;display (X=0)
        jsr     Disp_Msg
        jmp     cl_exit

cl_2:   sthx    intacc2+0
        jsr     udvd32

             ;result = uH x 10
        jsr     Bin2BCD               ;convert intacc1(2:3) to BCD in bcd10000_bcd_1000_bcd100_bcd10_bcd1
        lda     #$C3
        jsr     Cmnd2LCD
        lda     #'L'                  ;display 'L= '
        jsr     Data2LCD
        lda     #'='
        jsr     Data2LCD
        lda     #' '
        jsr     Data2LCD
        lda     bcd1000               ;display 1000's digit
        jsr     _nib_to_asc
        jsr     Data2LCD
        lda     bcd100                ;display 100's digit
        jsr     _nib_to_asc
        jsr     Data2LCD
        lda     bcd10                 ;display 10's digit
        jsr     _nib_to_asc
        jsr     Data2LCD
        lda     #'.'                  ;display decimal point
        jsr     Data2LCD
        lda     bcd1                  ;display 1's digit
        jsr     _nib_to_asc
        jsr     Data2LCD
        lda     #' '                  ;display ' uH'
        jsr     Data2LCD
        lda     #'u'
        jsr     Data2LCD
        lda     #'H'
        jsr     Data2LCD

             ;and exit
cl_exit:
        ldhx    cursor_index          ;put cursor back up on digit of freq being modified
        lda     digit_posn,X          ;get LCD digit address
        jsr     cmnd2LCD              ;and put cursor back up there on Freq line
        rts

Z_err2: fcb  '(Z>600)     ',0
X_err2: fcb  '(X=0)       ',0

;*****************************************************************
; Calc_C - Calculate the capacitance: C= 1/(6.28*F*X)
;          C = (10,000/63) x (10,000/kHz) x (10,000/X*10)
;                Factor_1  x   Factor_2   x   Factor_3

Calc_C:
        lda     Z_maxed               ;check for Z>600
        beq     cc_05                 ;it's less than, so proceed
        lda     #$C6
        jsr     Cmnd2LCD
        ldhx    #Z_err2               ;display (Z>600)
        jsr     Disp_Msg
        jmp     cc_exit

cc_05:  ldhx    Reactance             ;check for X < 1.0
        cphx    #$000A
        bcc     cc_1                  ;it's > that, so proceed
        lda     #$C3
        jsr     Cmnd2LCD
        lda     #'C'                  ;else display 'C= ' (X=0)
        jsr     Data2LCD
        lda     #'='
        jsr     Data2LCD
        lda     #' '
        jsr     Data2LCD
        ldhx    #X_err2               ;display (X=0)
        jsr     Disp_Msg
        jmp     cc_exit

            ;Compute Factor_1
cc_1:   clr     intacc1+0
        clr     intacc1+1
        mov     #$27,intacc1+2
        mov     #$10,intacc1+3
        clr     intacc2+0
        mov     #$3F,intacc2+1
        jsr     udvd32               ;10,000/63
        ldhx    #temp1_acc
        jsr     store_acc1

            ;Compute Factor_2
        jsr     GetFreq              ;get frequency dial into HX (kHz)
        cphx    #0                   ;check for F=0
        bne     cc_2                 ;F>0, so continue
        lda     #$C6                 ;F=0, so display (X=0)
        jsr     Cmnd2LCD
        ldhx    #X_err3              ;display (X=0)
        jsr     Disp_Msg
        jmp     cc_exit

cc_2:   sthx    intacc2+0
        clr     intacc1+0
        mov     #$01,intacc1+1
        mov     #$86,intacc1+2
        mov     #$A0,intacc1+3
        jsr     udvd32               ;100,000/kHz
        ldhx    #temp1_acc
        jsr     load_acc2
        jsr     umult32              ;Factor_1 x Factor_2
        ldhx    #temp1_acc
        jsr     store_acc2

            ;Compute Factor_3
        clr     intacc1+0
        clr     intacc1+1
        mov     #$27,intacc1+2
        mov     #$10,intacc1+3
        ldhx    Reactance
        sthx    intacc2+0
        jsr     udvd32                ; 10,000/(X*10)
        ldhx    #temp1_acc
        jsr     load_acc2
        jsr     umult32               ; (Factor_1 x Factor_2) * Factor_3
        mov     intacc2+0,intacc1+0
        mov     intacc2+1,intacc1+1
        mov     intacc2+2,intacc1+2
        mov     intacc2+3,intacc1+3

             ;result = pF x 100
        jsr     Bin2BCD               ;convert intacc1(2:3) to BCD in bcd100000_bcd10000_bcd_1000_bcd100_bcd10_bcd1
        lda     #$C3
        jsr     Cmnd2LCD
        lda     #'C'                  ;display 'C= '
        jsr     Data2LCD
        lda     #'='
        jsr     Data2LCD
        lda     #' '
        jsr     Data2LCD
        lda     bcd100000             ;display 100000's digit
        jsr     _nib_to_asc
        jsr     Data2LCD
        lda     bcd10000              ;display 10000's digit
        jsr     _nib_to_asc
        jsr     Data2LCD
        lda     bcd1000               ;display 1000's digit
        jsr     _nib_to_asc
        jsr     Data2LCD
        lda     bcd100                ;display 100's digit
        jsr     _nib_to_asc
        jsr     Data2LCD
        lda     #'.'                  ;display decimal point
        jsr     Data2LCD
        lda     bcd10                 ;display 10's digit
        jsr     _nib_to_asc
        jsr     Data2LCD
        lda     #' '                  ;display ' pF'
        jsr     Data2LCD
        lda     #'p'
        jsr     Data2LCD
        lda     #'F'
        jsr     Data2LCD

             ;and exit
cc_exit:
        ldhx    cursor_index          ;put cursor back up on digit of freq being modified
        lda     digit_posn,X          ;get LCD digit address
        jsr     cmnd2LCD              ;and put cursor back up there on Freq line
        rts

X_err3: fcb  '(X=0)       ',0

;*****************************************************************
; Calc_Batt - Calculate the battery voltage
;             5V max of 8-bit ADC = .0195 V/bit = .02 V/bit
;             Voltage divider: 10K/(22K+10K) = (.3125) x Vin

Calc_Batt:
        lda     #Chan_Batt
        jsr     Read_ADC              ;get reading of voltage divider

             ;Multiply reading x .02V/bit
             ;but scale up by 100 to handle denominator, to yield Vbattx100
        sta     intacc1+1
        clr     intacc1+0
        mov     #195,intacc2+1        ;multiply reading by 195, or (reading x (.0195 x 10,000))
        clr     intacc2+0
        jsr     umult16               ;intacc1(0:1) x intacc2(0:1) = intacc1(0:3)

             ;Multiply Vbattx100 by 10,000 (so we can divide by 3125 to get Vin)
        clr     intacc2+0
        clr     intacc2+1
        mov     #$27,intacc2+2        ;10,000. = $2710
        mov     #$10,intacc2+3
        jsr     umult32               ;intacc1(0:3) x intacc2(0:3) = intacc1(0:3)...intacc2(0:3)

             ;Lastly, divide by 3125 to get Vin
        mov     intacc2+0,intacc1+0   ;move big number to intacc1
        mov     intacc2+1,intacc1+1
        mov     intacc2+2,intacc1+2
        mov     intacc2+3,intacc1+3
        mov     #$0C,intacc2+0        ;3125. = $0C35
        mov     #$35,intacc2+1
        jsr     udvd32                ;intacc1(0:3) / intacc2(0:1) = intacc1(0:3)
        jsr     Bin2BCD
        jsr     dpr_acc1              ;debug print Vbatt
        rts


;***************************************************************************
; Bin2BCD - Convert intacc1(2:3) to BCD in bcd100,bcd10,bcd1

Bin2Bcd:
        pshhx
        lda     intacc1+3       ;get 16 bit binary number into HX
        tax
        lda     intacc1+2
        tah
        lda     intacc1+1       ;check next higher byte for a $01 (max we'll see)
        beq     b2b3            ;0 = less than 65,535, so start counting as such
        cmp     #$01
        beq     b2b65           ;1 = start counting at 65,535
        cmp     #$02
        beq     b2b131          ;2 = start counting at 131,070
        cmp     #$03
        beq     b2b196          ;3 = start counting at 196,605
        bra     b2b3            ;should never get here

             ;number is 02xxxx, so start counting bcd at 196,605
b2b196: mov     #1,bcd100000
        mov     #9,bcd10000
        mov     #6,bcd1000
        mov     #6,bcd100
        mov     #0,bcd10
        mov     #5,bcd1
        bra     b2b2
             ;number is 01xxxx, so start counting bcd at 131,070
b2b131: mov     #1,bcd100000
        mov     #3,bcd10000
        mov     #1,bcd1000
        mov     #0,bcd100
        mov     #7,bcd10
        mov     #0,bcd1
        bra     b2b2
             ;number is 01xxxx, so start counting bcd at 65,535
b2b65:  clr     bcd100000
        mov     #6,bcd10000
        mov     #5,bcd1000
        mov     #5,bcd100
        mov     #3,bcd10
        mov     #5,bcd1
        bra     b2b2
             ;number is less than 65,535 (FFFF), so clear out bcd regs
             ;for starting count at 00,000.
b2b3:   clr     bcd100000
        clr     bcd10000
        clr     bcd1000
        clr     bcd100
        clr     bcd10
        clr     bcd1

             ;start counting down HX by 10 while incrementing bcd regs by 10
b2b2:   cphx    #0              ;result =0?
        beq     b2b_exit        ;yes, done
        cphx    #10             ;result < 10?
        bcs     b2b_exit        ;yes, done
b2b1:   aix     #-10            ;bin - 10
        inc     bcd10           ;no, just increment BCD counters
        lda     bcd10
        cmp     #10             ;rollover on the 10s digit?
        bne     b2b2            ;no, do again
        clr     bcd10           ;yes, rollover
        inc     bcd100
        lda     bcd100
        cmp     #10             ;rollover on the 100s digit?
        bne     b2b2            ;no, do again
        clr     bcd100          ;yes, rollover
        inc     bcd1000
        lda     bcd1000
        cmp     #10             ;rollover on the 1000s digit?
        bne     b2b2            ;no, do again
        clr     bcd1000         ;yes, rollover
        inc     bcd10000
        lda     bcd10000
        cmp     #10             ;rollover on the 10000s digit?
        bne     b2b2            ;no, do again
        clr     bcd10000
        inc     bcd100000
        bra     b2b2            ;and do again

b2b_exit:
        txa
        sta     bcd1            ;save "less than 10 remainder" in bcd1
        pulhx
        rts

;***********************************************************************
; Test Square Root
             ;Test: SQRT(115) with guess of (3.3 x 10)
             ;Answer = 6B = 107 = "10.7"
test_sqrt:
        ldhx    #33                     ;initial guess x10 (assuming scale-up of number by 100)
        clr     intacc1+0               ;number to be rooted (internally scaled up by 100)
        clr     intacc1+1
        clr     intacc1+2
        mov     #115,intacc1+3
        jsr     sq_root
        jsr     dpr_acc1                ;Debug print SQRT
        jsr     dpr_crlf
        rts
