;********************************************************************************
; Calibrate -- Method B
;
; Prompt the user though several steps while taking readings of Vf, Vr, Vz and Va to
; determine what "correction factor" (multiplication ratio) is needed to make the
; run-time readings correct.
;
; We first need to set gain with a high impedance load on the Analyzer since
; we still need to measure SWR with high loads and those readings can be unpredictable
; if the ADC gets a too-high input. Do this at 1.8 MHz.
;
;1. With an open circuit, set the VR chain to get the 225 to 250.
;2. With a short circuit, measure Vf for reference then adjust the Va chain to get
;   the same reading (debug mode).
;3. With the same short circuit, set Vr for the same reading.
;4. Put on a 50 ohm load and measure VF for reference, then adjust the
;   Vz chain to get exactly half the Vr reading.

Calibrate:
        jsr     Set_DDS_off             ;start with no DDS signal applied
        lda     #0
        sta     EE_dirty                ;clear flag first to not store new values in EEPROM
        sta     Mode_nr                 ;select the Impedance Mode
;        sta     DDS_active              ;clear flag so not to display SWR or '+j'
        jsr     LCD_clear
        ldhx    #CalMsg1                ;"Calibrate now?"
        jsr     Disp_Msg
        jsr     Get_YN                  ;get Yes/No answer
        bcs     cal1                    ;CY=1=Yes
        jmp     Config_Process          ;no, just exit
cal1:   jsr     LCD_Clear
        ldhx    #CalMsg2                ;"Open load?"
        jsr     Disp_Msg
        jsr     Get_YN                  ;get Yes/No answer
        bcs     cal1a                   ;CY=1=Yes
        jmp     Config_Process          ;no, just exit

            ;--------------------
            ;RF CONNECTOR IS OPEN
            ;Get offset readings (with no signal applied)
            ;--------------------
cal1a:  jsr     wait100ms
        lda     #CHAN_Vf
        jsr     read_ADC                ;read Vf offset
        sta     Vf_offset

        lda     #CHAN_Vr
        jsr     read_ADC                ;read Vr offset
        sta     Vr_offset

        lda     #CHAN_Vz                ;read Vz offset
        jsr     read_ADC
        sta     Vz_offset

        lda     #CHAN_Va                ;read Va
        jsr     read_ADC
        sta     Va_offset

            ;--------------------
            ;RF CONNECTOR IS OPEN
            ;All four ADC readings need to be less than FF
            ;Adjust DDS trimpot such that all are < FA, then press Dial to continue
            ;--------------------
cal2:
        jsr     Set_10Mhz               ;do open load check at 10 MHz
        jsr     LCD_Clear
        ldhx    #ADC_Names              ;Display "VF  VR  VZ  VA" on first line
        jsr     Disp_Msg

dadc_1: lda     #$C0                    ;point to spot on 2nd line
        jsr     Cmnd2LCD
        lda     #CHAN_Vf
        jsr     read_ADC                ;read Vf
        jsr     disp_ADC

        lda     #$C4                    ;point to spot on 2nd line
        jsr     Cmnd2LCD
        lda     #CHAN_Vr
        jsr     read_ADC                ;read Vr
        jsr     disp_ADC

        lda     #$C8                    ;point to spot on 2nd line
        jsr     Cmnd2LCD
        lda     #CHAN_Vz                ;read Vz
        jsr     read_ADC
        jsr     disp_ADC

        lda     #$CC                    ;point to spot on 2nd line
        jsr     Cmnd2LCD
        lda     #CHAN_Va                ;read Va
        jsr     read_ADC
        jsr     disp_ADC

        brset   Enc_PB,portF,dadc_1     ;continue looping until ENC PB pressed
        brclr   Enc_PB,portF,*          ;await release of ENC PB

             ;proceed on to next step
        jsr     LCD_Clear
        ldhx    #CalMsg3                ;"Shorted load?"
        jsr     Disp_Msg
        jsr     Get_YN                  ;get Yes/No answer
        bcs     cal3                    ;CY=1=Yes
        jmp     Config_Process          ;no, just exit

            ;--------------------------------------
            ;RF CONNECTOR IS SHORTED.
            ;    Read Vr and determine factor to make it the same as full scale Vfx100.
            ;    Read Va and determine factor to make it the same as full scale Vfx100.
            ;---------------------------------------
cal3:
        jsr     _crlf
        lda     #CHAN_Vf
        jsr     read_ADC              ;read Vf
        jsr     print_A_nocr          ;print raw Vf
        sta     intacc1+3
        clr     intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        lda     Vf_offset             ;subract offset
        jsr     print_A_nocr          ;print Vf offset
        sta     intacc2+3
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     usub32                ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        lda     intacc1+3
        jsr     print_A_nocr          ;print offset-adjusted Vf
        sta     intacc2+1             ;store Vf in intacc2(0:1)
        clr     intacc2+0
        mov     #$64,intacc1+1
        clr     intacc1+0
        jsr     umult16               ;create Vfx100
        ldhx    #Vfx100
        jsr     store_acc1
        jsr     print_acc1            ;print Vfx100

            ;Determine Va correction factor

        lda     #CHAN_Va
        jsr     read_ADC              ;read Va
        jsr     print_A_nocr          ;print raw Va
        sta     intacc1+3
        clr     intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        lda     Va_offset             ;subract offset
        jsr     print_A_nocr          ;print Va offset
        sta     intacc2+3
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     usub32                ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        lda     intacc1+3
        jsr     print_A_nocr          ;print offset-adjusted Va
        sta     intacc2+1
        clr     intacc2+0
        ldhx    #Vfx100
        jsr     load_acc1             ;perform Vfx100 / Va to determine Va_corr
        jsr     udvd32                ;intacc1(0:3) / intacc2(0:1) = intacc1(0:3)
        lda     intacc1+3
        sta     Va_corrfact
        jsr     print_A_nocr          ;print Va correction factor
        lda     #'r'
        jsr     _putch                ;print 'r' for remainder that follows
        lda     intacc2+1             ;get remainder
        sta     Va_corrfact+1         ;and store remainder
        jsr     print_A               ;and print remainder

            ; Determine Vr correction factor

        lda     #CHAN_Vr
        jsr     read_ADC              ;read Vr
        jsr     print_A_nocr          ;print raw Vr
        sta     intacc1+3
        clr     intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        lda     Vr_offset             ;subract offset
        jsr     print_A_nocr          ;print Vr offset
        sta     intacc2+3
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     usub32                ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        lda     intacc1+3
        jsr     print_A_nocr          ;print offset-adjusted Vr
        sta     intacc2+1
        clr     intacc2+0
        ldhx    #Vfx100
        jsr     load_acc1             ;perform Vfx100 / Vr to determine Vr_corr
        jsr     udvd32                ;intacc1(0:3) / intacc2(0:1) = intacc1(0:3)
        lda     intacc1+3
        sta     Vr_corrfact
        jsr     print_A_nocr          ;print Va correction factor (1st)
        lda     #'r'
        jsr     _putch                ;print 'r' for remainder that follows
        lda     intacc2+1             ;get remainder
        sta     Vr_corrfact+1         ;and store remainder
        jsr     print_A               ;and print remainder

        jsr     LCD_Clear
        ldhx    #CalMsg5              ;"270-ohm load on?"
        jsr     Disp_Msg
        jsr     Get_YN                ;get Yes/No answer
        bcs     cal3a                 ;CY=1=Yes
        jmp     Config_Process        ;no, just exit

cal3a:  jsr     LCD_Clear
        ldhx    #CalMsg4              ;"Make SWR=5.4"
        jsr     Disp_Msg

cal3b:  jsr     Calc_SWR
        lda     #1                    ;dial sensitivity
        jsr     Read_Encoder          ;process the encoder turns
        bcs     cal3d                 ;Dial press = exit
        bne     cal3c
        lda     Vr_corrfact           ;CW turn detected ... increase Vr_corrfactor
        inca
        sta     Vr_corrfact
        bra     cal3b
cal3c:
        lda     Vr_corrfact           ;CCW turn detected ... decrease Vr_corrfactor
        deca
        sta     Vr_corrfact
        bra     cal3b
cal3d:
        lda     Vr_corrfact
        jsr     print_A_nocr          ;print Vr correction factor (final)
        lda     #'r'
        jsr     _putch                ;print 'r' for remainder that follows
        lda     intacc2+1             ;get remainder
        sta     Vr_corrfact+1         ;and store remainder
        jsr     print_A               ;and print remainder
        brclr   ENC_PB,portF,*        ;await release of Encoder PB

             ;Determine correction factor for Vz to be 1/2 Vf (using 50-ohm load)

cal3e:  jsr     LCD_clear
        ldhx    #CalMsg50a             ;"50-ohm load on?"
        jsr     Disp_Msg
        jsr     Get_YN
        bcs     cal3f                  ;CY=1=Yes
        jmp     Config_Process         ;no, just exit

cal3f:  lda     #CHAN_Vf
        jsr     read_ADC              ;read Vf
        jsr     print_A_nocr          ;print Vf raw (50)
        sta     intacc1+3
        clr     intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        lda     Vf_offset             ;subract offset
        jsr     print_A_nocr          ;print Vf offset
        sta     intacc2+3
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     usub32                ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        lda     intacc1+3
        jsr     print_A_nocr          ;print Vf_offset_adjusted (50)
        mov     intacc1+3,intacc2+1   ;get adjusted Vf into intacc2(0:1)
        mov     intacc1+2,intacc2+0
        mov     #$32,intacc1+1
        clr     intacc1+0
        jsr     umult16               ;create Vfx50
        ldhx    #Vfx100
        jsr     store_acc1
        jsr     print_acc1            ;print Vfx50 (50)
        lda     #CHAN_Vz
        jsr     read_ADC              ;read Vz
        jsr     print_A_nocr          ;print raw Vz (50)
        sta     intacc1+3
        clr     intacc1+2
        clr     intacc1+1
        clr     intacc1+0
        lda     Vz_offset             ;subract offset
        jsr     print_A_nocr          ;print Vz offset
        sta     intacc2+3
        clr     intacc2+2
        clr     intacc2+1
        clr     intacc2+0
        jsr     usub32                ;intacc1(0:3) - intacc2(0:3) = intacc1(0:3)
        lda     intacc1+3
        jsr     print_A_nocr          ;print Vz_offset_adj (50)
        sta     intacc2+1
        clr     intacc2+0
        ldhx    #Vfx100
        jsr     load_acc1
        jsr     udvd32                ;intacc1(0:3) / intacc2(0:1) = intacc1(0:3)
        lda     intacc1+3
        sta     Vz_corrfact
        jsr     print_A_nocr          ;print Vz correction factor
        lda     #'r'
        jsr     _putch                ;print 'r' for remainder that follows
        lda     intacc2+1             ;get remainder
        sta     Vz_corrfact+1         ;and store remainder
        jsr     print_A               ;and print remainder

            ; Calibration Complete!
        jsr     LCD_clear
        ldhx    #CalDone                ;"Cal complete!"
        jsr     Disp_Msg
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    #Save_Settings
        jsr     disp_msg                ;"Saving settings.."
        bclr    BusyLED,portc           ;turn on Busy LED
        jsr     Erase_EEPROM_P1         ;erase EEPROM Page 1
        jsr     Put_EEPROM_P1           ;write Cal Data to EEPROM Page 1
        bset    BusyLED,portc           ;turn off Busy LED
        jmp     Config_Process

            ;----------------
            ;Store intacc1(0:3) at H:X
store_acc1:
        lda     intacc1+0
        sta     0,x
        lda     intacc1+1
        sta     1,x
        lda     intacc1+2
        sta     2,x
        lda     intacc1+3
        sta     3,x
        rts

            ;----------------
            ;Store intacc2(0:3) at H:X
store_acc2:
        lda     intacc2+0
        sta     0,x
        lda     intacc2+1
        sta     1,x
        lda     intacc2+2
        sta     2,x
        lda     intacc2+3
        sta     3,x
        rts

           ;--------------------------
           ;Load intacc1(0:3) from H:X
load_acc1:
        lda     0,x
        sta     intacc1+0
        lda     1,x
        sta     intacc1+1
        lda     2,x
        sta     intacc1+2
        lda     3,x
        sta     intacc1+3
        rts

           ;--------------------------
           ;Load intacc2(0:3) from H:X
load_acc2:
        lda     0,x
        sta     intacc2+0
        lda     1,x
        sta     intacc2+1
        lda     2,x
        sta     intacc2+2
        lda     3,x
        sta     intacc2+3
        rts


            ;-----------------------------------------------------
            ;Disp_ADC
            ;display the ADC reading in A as two ascii characters
Disp_ADC:
        jsr     _dblchar                ;convert reg A to two equiv ascii chars in HX
        tha
        jsr     data2LCD
        txa
        jsr     data2LCD
        rts

           ;--------------------------
           ;SetDDS Routines

Set_DDS_Off:
        ldhx    #DDS_off                ;set DDS to 0 Hz
        bra     setDDSregs

Set_1p8Mhz:
        ldhx    #DDS_1p8MHz             ;set DDS to 180m
        bra     setDDSregs

Set_3p7Mhz:
        ldhx    #DDS_3p7MHz             ;set DDS to 80m
        bra     setDDSregs

Set_7Mhz:
        ldhx    #DDS_7MHz               ;set DDS to 40m
        bra     setDDSregs

Set_10Mhz:
        ldhx    #DDS_10MHz              ;set DDS to 30m
        bra     setDDSregs

Set_12Mhz:
        ldhx    #DDS_12MHz              ;set DDS to 24m
        bra     setDDSregs

Set_15Mhz:
        ldhx    #DDS_15MHz              ;set DDS to 20m
        bra     setDDSregs

Set_18Mhz:
        ldhx    #DDS_18MHz              ;set DDS to 18m
        bra     setDDSregs

Set_21Mhz:
        ldhx    #DDS_21MHz              ;set DDS to 15m
        bra     setDDSregs

Set_24Mhz:
        ldhx    #DDS_24MHz              ;set DDS to 12m
        bra     setDDSregs

Set_28Mhz:
        ldhx    #DDS_28MHz              ;set DDS to 10m
        bra     setDDSregs

SetDDSregs:
        lda     0,x
        sta     DDS_w0
        lda     1,x
        sta     DDS_w1
        lda     2,x
        sta     DDS_w2
        lda     3,x
        sta     DDS_w3
        lda     4,x
        sta     DDS_w4
        lda     5,x
        sta     LCD_Dat+0                ;load up 10M digit into LCD display register
        lda     6,x
        sta     LCD_Dat+1                ;load up 1M digit into LCD display register
        lda     7,x
        sta     LCD_Dat+2                ;load up 100K digit into LCD display register
        clr     LCD_Dat+3
        clr     LCD_Dat+4
        clr     LCD_Dat+5
        clr     LCD_Dat+6
        jsr     Set_DDS
        jsr     Set_DDS
        jsr     Set_DDS
        rts

DDS_off:     fcb   0,0,0,0,0
             fcb   0,0,0
DDS_1p8MHz:  fcb   $00,$04,$9B,$A5,$E3
_1p8MHz:     fcb   0,1,8
DDS_3p7MHz:  fcb   $00,$09,$78,$D4,$FE
_3p7MHz:     fcb   0,3,7
DDS_7MHz:    fcb   $00,$11,$EB,$85,$1F
_7MHz:       fcb   0,7,0
DDS_10MHz:   fcb   $00,$19,$99,$99,$9A
_10MHz:      fcb   1,0,0
DDS_12MHz:   fcb   $00,$1E,$B8,$51,$EC
_12MHz:      fcb   1,2,0
DDS_15MHz:   fcb   $00,$26,$66,$66,$66
_15MHz:      fcb   1,5,0
DDS_18MHz:   fcb   $00,$2A,$14,$7A,$E1
_18MHz:      fcb   1,8,0
DDS_21MHz:   fcb   $00,$35,$C2,$8F,$5C
_21MHz:      fcb   2,1,0
DDS_24MHz:   fcb   $00,$3D,$70,$A3,$D7
_24MHz:      fcb   2,4,0
DDS_28MHz:   fcb   $00,$47,$AE,$14,$7B
_28MHz:      fcb   2,8,0

ADC_Names:   fcb     'VF  VR  VZ  VA  ',0
CalMsg1:     fcb     'Calibrate now?',0
CalMsg2:     fcb     'Open load?',0
CalMsg3:     fcb     'Shorted load?',0
CalMsg4:     fcb     'Make SWR=5.4',0
CalMsg5:     fcb     '270-ohm load on?',0
CalMsg50a:   fcb     '50-ohm load on?',0
CalDone:     fcb     'Cal complete!',0
