;*********************************************************************************
;*********************************************************************************
;                       Config Process for Micro908
;*********************************************************************************
;*********************************************************************************

Config_Process:
        brclr   ConfigPB,portG,*        ;wait until Config PB released
        tone_off                        ;turn tone off while in Config menu
        jsr     LCD_Clear
        ldhx    #Config_msg
        jsr     disp_msg                ;display "Setup Selection" on 1st line

        ldhx    #ConfigCmndTable        ;set up pointer to ConfigCmndTable
        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #ConfigMsgTable         ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
        sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection

             ;Save all Config settings in EEPROM if EE_dirty flag is set, then exit
exitconfig:
        lda     EE_dirty
        beq     cp_exit                 ;just exit if no settings changes made
        bclr    BusyLED,portc           ;turn on Busy LED
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    #Save_Settings
        jsr     disp_msg                ;"Saving settings.."
        jsr     SaveUserSettings
        bset    BusyLED,portc           ;turn off Busy LED
        brclr   ENC_PB,portf,*          ;wait here until Encoder PB released
cp_exit:
        clra
        sta     EE_dirty
        jmp     Impedance

Config_msg:     fcb     'Config Selection',0
SetStep_msg:    fcb     'Select step size:',0
SetCustom_msg:  fcb     '- Custom Band -',0
Save_Settings:  fcb     '    (saving)   ',0

;-----------------------------
; SaveUserSettings -- Saves users settings to EEPROM.

SaveUserSettings:
        bclr    BusyLED,portc           ;turn on Busy LED
        jsr     Put_EEPROM_P0           ;Save Config Data to EEPROM Page 0
                                        ;(routine located in Drivers_m.asm)
        bset    BusyLED,portc           ;turn off Busy LED
        rts

;--------------------------
;   Config JUMP TABLE

ConfigCmndTable:
        fdb     SetStepSize
        fdb     ExitConfig
        fdb     Calibrate
        fdb     Debug_Mode
        fdb     Monitor
        fdb     DispBatt
        fdb     Default_Settings
        fdb     DataOptions
        fdb     ViewSettings
        fdb     ToneOff
        fdb     ToneOn
        fdb     SetCustomEnd
        fdb     SetCustomStart

;---------------------------
;   Config MESSAGE TABLE

ConfigMsgTable:
        fcb     'Step Size',0           ;CW direction
        fcb     'Exit',0                ;1st message displayed
        fcb     'Calibrate',0           ;CCW direction
        fcb     'DEBUG Mode',0
        fcb     'Software Load',0
        fcb     'Battery Voltage',0
        fcb     'Default_Settings',0
        fcb     'Data Options',0
        fcb     'View Settings',0
        fcb     'Disable Tones',0
        fcb     'Enable Tones',0
        fcb     'End Freq',0
        fcb     'Start Freq',0
        fcb     0

;******************************************************************************
; DispBatt -- Display the battery voltage

DispBatt:
        bsr     dispbatt1
        brset   ENC_PB,portf,*          ;wait here until Encoder PB released
        brclr   ENC_PB,portf,*          ;wait here until Encoder PB released
        jmp     Config_Process


DispBatt1:
        jsr     LCD_Clear
        ldhx    #Batt_msg               ;"Battery Voltage:"
        jsr     Disp_Msg
        jsr     Calc_Batt
        lda     #$89                    ;point to mid-1st line
        jsr     Cmnd2LCD

             ;display battery voltage on second line
        lda     bcd100000
        add     #$30
        jsr     data2lcd
        lda     bcd10000
        add     #$30
        jsr     data2lcd
        lda     #'.'
        jsr     data2lcd
        lda     bcd1000
        add     #$30
        jsr     data2lcd
        lda     bcd100
        add     #$30
        jsr     data2lcd
        rts

Batt_msg:  fcb  'Battery:',0

;*********************************************************************************
; SetStepSize --
;      Sets 'StepPtr" to one of the four valid offsets into the LCD_Dat display array:
;            LCD_Dat+0, LCD_Dat+1, LCD_Dat+2, LCD_Dat+3, LCD_Dat+4, LCD_Dat+5, LCD_Dat+6
;              10 MHz     1 MHz     100 kHz     10 kHz     1 kHz     100 Hz      10 Hz
;                           StepPtr =  02        03         04        05         06

SetStepSize:
        jsr     LCD_Clear
        ldhx    #SetStep_msg
        jsr     disp_msg                ;display "Select step size:" on 1st line

        ldhx    #StepCmndTable          ;set up pointer to ConfigCmndTable
        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #StepMsgTable           ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
        sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jsr     get_dial_selection
        jmp     Config_Process

Sel_100KHz:
        ldhx    #$02
        bra     sss_exit

Sel_10KHz:
        ldhx    #$03
        bra     sss_exit

Sel_1KHz:
        ldhx    #$04
        bra     sss_exit

Sel_100Hz:
        ldhx    #$05
        bra     sss_exit

Sel_10Hz:
        ldhx    #$06

sss_exit:
        sthx    StepPtr
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;--------------------------
;   Step JUMP TABLE

StepCmndTable:
        fdb     Sel_100KHz
        fdb     Sel_10KHz
        fdb     Sel_1KHz
        fdb     Sel_100Hz
        fdb     Sel_10Hz

;---------------------------
;   Step MESSAGE TABLE

StepMsgTable:
s2:     fcb     '100 kHz',0
s3:     fcb     '10 kHz',0
s4:     fcb     '1 kHz',0
s5:     fcb     '100 Hz',0
s6:     fcb     '10 Hz',0
        fcb     0

;*********************************************************************************
SetCustomStart:
        jsr     LCD_Clear
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Start_msg               ;"START frequency:"
        jsr     Disp_Msg
        clr     DDS_active               ;clear flag to not use DDS or SWR calcs

        mov     Custom_Start+0,LCD_dat+0 ;start the display at previous (or default)
        mov     Custom_Start+1,LCD_dat+1 ;  custom start setting
        mov     Custom_Start+2,LCD_dat+2
        mov     Custom_Start+3,LCD_dat+3
        mov     Custom_Start+4,LCD_dat+4
        mov     Custom_Start+5,LCD_dat+5
        mov     Custom_Start+6,LCD_dat+6

        ldhx    #2
        sthx    cursor_index             ;digit address for cursor

        jsr     Process_Dial             ;put current Custom_Start to the LCD
                                         ; and allow user to modify it

        mov     LCD_dat+0,Custom_Start+0 ;save selected start frequency
        mov     LCD_dat+1,Custom_Start+1
        mov     LCD_dat+2,Custom_Start+2
        mov     LCD_dat+3,Custom_Start+3
        mov     LCD_dat+4,Custom_Start+4
        mov     LCD_dat+5,Custom_Start+5
        mov     LCD_dat+6,Custom_Start+6
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

Start_msg: fcb  '(exit = <Config>)',0

;*********************************************************************************
SetCustomEnd:
        jsr     LCD_Clear
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #End_msg                 ;"END:"
        jsr     Disp_Msg
        clr     DDS_active               ;clear flag to not use DDS or SWR calcs

        mov     Custom_End+0,LCD_dat+0   ;start the display at previous (or default)
        mov     Custom_End+1,LCD_dat+1   ;  custom end setting
        mov     Custom_End+2,LCD_dat+2
        mov     Custom_End+3,LCD_dat+3
        mov     Custom_End+4,LCD_dat+4
        mov     Custom_End+5,LCD_dat+5
        mov     Custom_End+6,LCD_dat+6

        ldhx    #2
        sthx    cursor_index             ;digit address for cursor

        jsr     Process_Dial             ;put current Custom_End to the LCD
                                         ; and allow user to modify it

        mov     LCD_dat+0,Custom_End+0   ;save selected start frequency
        mov     LCD_dat+1,Custom_End+1
        mov     LCD_dat+2,Custom_End+2
        mov     LCD_dat+3,Custom_End+3
        mov     LCD_dat+4,Custom_End+4
        mov     LCD_dat+5,Custom_End+5
        mov     LCD_dat+6,Custom_End+6
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

End_msg:   fcb  '(exit = <Config>)',0

;********************************************************************************
ToneOn:
        lda     #1
        sta     Tone_Flag
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;********************************************************************************
ToneOff:
        clra
        sta     Tone_Flag
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;*********************************************************************************
; DataOptions --
;      Allows user to:
;             Not save data from Scan
;             Save data from Scan to SEEPROM
;             Send data during Scan
;             Send data from SEEPROM now

DataOptions:
        jsr     LCD_Clear
        ldhx    #Data_msg
        jsr     disp_msg                ;display "Data Options:" on 1st line

        ldhx    #DataCmndTable          ;set up pointer to ConfigCmndTable
        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #DataMsgTable           ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
        sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jsr     get_dial_selection
        jmp     Config_Process

nd:     lda     #NoData
        bra     do_exit

sds:    lda     #SendDuringScan
        bra     do_exit

sfm:    lda     #SendFromMem
        bra     do_exit

            ; Send data from SEEPROM to PC
d2m:    nop                             ;INSERT SEEPROM USAGE CODE HERE
        lda     #Data2Mem

do_exit:
        sta     DataMode
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;--------------------------
;   Data JUMP TABLE

DataCmndTable:
        fdb     sfm                     ;SendFromMem
        fdb     nd                      ;NoData
        fdb     d2m                     ;Data2Mem
        fdb     sds                     ;SendDuringScan

;---------------------------
;   Data MESSAGE TABLE

DataMsgTable:
        fcb     'Send from Mem',0       ;CW
        fcb     'No Data Save',0        ;start
        fcb     'Data to Mem',0         ;CCW
        fcb     'Send during Scan',0
        fcb     0

Data_msg:   fcb  'Data Options:',0

;********************************************************************************
; Default Settings
; Returns user-customizable settings to default values in EEPROM

Default_Settings:
        jsr     LCD_clear
        ldhx    #Default_Msg1         ;"Restore default?"
        jsr     Disp_Msg
        jsr     Get_YN                ;get Yes/No answer
        bcs     ds1                   ;CY=1=Yes
        clra
        sta     debug_flag            ;CY=0=No, so turn off flag
        clr     EE_dirty              ;don't save settings upon exiting Config
        jmp     Config_Process        ;and exit

            ;Clear EEPROM
ds1:    jsr     LCD_clear
        ldhx    #Default_Msg2         ;'(please wait)'
        jsr     Disp_Msg
        bclr    BusyLED,portc         ;turn on Busy LED
        jsr     Erase_EEPROM_P0       ;erase EEPROM Page 0 locations
        jsr     Init_EEPROM_vars      ;set nonvolatile vars to default settings
        jsr     Put_EEPROM_P0a        ;write nonvolatile vars to EEPROM Page 0
        bset    BusyLED,portc         ;turn off Busy LED
        jmp     Config_Process

Default_Msg1:  fcb  'Restore default?',0
Default_Msg2:  fcb  ' (please wait) ',0

;********************************************************************************
; Debug Mode
; Turns Debug_Flag on/off to allow/disallow diagnostic print statements going
; to terminal

Debug_Mode:
        jsr     LCD_clear
        ldhx    #Debug_Msg1          ;"Debug Mode?"
        jsr     Disp_Msg
        jsr     Get_YN                ;get Yes/No answer
        bcs     dm1                   ;CY=1=Yes
        clra
        sta     debug_flag            ;CY=0=No, so turn off flag
        clr     EE_dirty              ;don't save settings upon exiting Config
        jmp     Config_Process        ;and exit

            ;Turn on Debug_Flag
dm1:    lda     #1
        sta     debug_flag
        clr     EE_dirty              ;don't save settings upon exiting Config
        jmp     Config_Process        ;and exit

Debug_Msg1:
        fcb     'Debug Mode?',0

;********************************************************************************
; Debug Monitor

Monitor:
        swi                           ; Software Interrupt enters HCmon
        rts

;*********************************************************************************
; Get Yes/No Response -- Presents Yes/No message selection to the LCD.
;                        Return with CY=1 if Yes
;                                    CY=0 if No

Get_YN:
        ldhx    #YNCmndTable            ;set up pointer to ConfigCmndTable
        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #YNMsgTable             ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
        sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection

Do_Yes: sec                             ;set CY = 1 = Yes
        rts                             ;this returns to the caller of Get_YN

Do_No:  clc                             ;clear CY = No
        rts                             ;this returns to the caller of Get_YN

;--------------------------
;   Yes/No JUMP TABLE

YNCmndTable:
        fdb     Do_no
        fdb     Do_Yes


;---------------------------
;   Yes/No MESSAGE TABLE

YNMsgTable:
No_ssg:  fcb     '  No',0
Yes_msg: fcb     '  Yes',0
         fcb     0

;********************************************************************************
ViewSettings:
        jsr     LCD_Clear
        ldhx    #UserSettings_msg       ;"User Settings:"
        jsr     Disp_Msg

             ;Display Custom_Start frequency

vs0:    jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Startf_msg             ;"Start:"
        jsr     Disp_Msg
        lda     #$C7
        ldhx    #Custom_Start
        jsr     Disp_Ufreq

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs05                    ;contnue if Enc not pressed
        jmp     vs_exit                 ;exit if Encoder PB pressed

             ;Display Custom_End frequency

vs05:   jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Endf_msg               ;"End:"
        jsr     Disp_Msg
        lda     #$C5
        ldhx    #Custom_End
        jsr     Disp_Ufreq

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs3
        jmp     vs_exit                 ;exit if Encoder PB pressed

             ;Display Step Size

vs3:    jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Stepf_msg              ;"Step:"
        jsr     Disp_Msg
        lda     #$C6
        jsr     Cmnd2LCD
        lda     StepPtr+1
        cmp     #2
        bne     vs3a
        ldhx    #s2                     ;"100 kHz"
        bra     vs7
vs3a:   cmp     #3
        bne     vs4
        ldhx    #s3                     ;"10 kHz"
        bra     vs7
vs4:    cmp     #4
        bne     vs5
        ldhx    #s4                     ;"1 kHz"
        bra     vs7
vs5:    cmp     #5
        bne     vs6
        ldhx    #s5                     ;"100 Hz"
        bra     vs7
vs6:    cmp     #6
        beq     vs6a
        jmp     vs_err
vs6a:   ldhx    #s6                     ;"10 Hz"

vs7:    jsr     disp_msg                ;display the Step size

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs7a                    ;continue
        jmp     vs_exit                 ;exit if Encoder PB pressed

             ;Display Tone setting

vs7a:   jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Tone_msg               ;"Tone:"
        jsr     Disp_Msg
        lda     #$C6
        jsr     Cmnd2LCD
        tst     Tone_Flag
        beq     vs7b
        ldhx    #ToneOn_msg             ;'enabled'
        bra     vs8
vs7b:   ldhx    #ToneOff_msg            ;'disabled'
vs8:    jsr     Disp_Msg

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs9                     ;continue
        jmp     vs_exit                 ;exit if Encoder PB pressed

             ;Display the DataMode

vs9:    jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #DataMode_msg           ;"DataMode:"
        jsr     Disp_Msg
        lda     #$CA
        jsr     Cmnd2LCD
        lda     DataMode
        cmp     #NoData
        bne     vs9a
        ldhx    #nd_msg                 ;"NoSend"
        bra     vs9c
vs9a:   cmp     #Data2Mem
        bne     vs9b
        ldhx    #d2m_msg                ;"To Mem"
        bra     vs9c
vs9b:   ldhx    #sds_msg                ;"To PC"
vs9c:   jsr     Disp_Msg
        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs12                    ;continue
        jmp     vs_exit                 ;exit if Encoder PB pressed

             ;Display Debug setting

vs12:   jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Debug_msg               ;"Debug:"
        jsr     Disp_Msg
        lda     #$CD
        jsr     Cmnd2LCD
        tst     Tone_Flag
        beq     vs10
        ldhx    #On_msg                 ;'on'
        bra     vs11
vs10:   ldhx    #Off_msg                ;'off'
vs11:   jsr     Disp_Msg

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcs     vs_exit                 ;exit if Encoder PB pressed

        jmp     vs0                     ;go loop again

vs_exit:
        brclr   Enc_PB,portf,*          ;await Encoder PB release
        jmp     Config_Process

vs_err: swi                             ;error if get here

;--------------------------
Disp_Ufreq:
        jsr     Cmnd2LCD                ;put cursor to selected line 2 spot
        lda     ,x                      ;display 10M digit
        ora     #$30
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 1M digit
        ora     #$30
        jsr     Data2LCD
        lda     #','                    ;display comma
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 100K digit
        ora     #$30
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 10K digit
        ora     #$30
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 1K digit
        ora     #$30
        jsr     Data2LCD
        lda     #'.'                    ;display decimal point
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 100 Hz digit
        ora     #$30
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 10 Hz digit
        ora     #$30
        jsr     Data2LCD
        rts

UserSettings_msg:  fcb  'User Settings:',0
Startf_msg:        fcb  'Start: ',0
Endf_msg:          fcb  'End: ',0
Stepf_msg:         fcb  'Step: ',0
Tone_msg:          fcb  'Tone: ',0
ToneOn_msg:        fcb  'enabled',0
ToneOff_msg:       fcb  'disabled',0
Debug_msg:         fcb  'Debug: ',0
On_msg:            fcb  'on',0
Off_msg:           fcb  'off',0
DataMode_msg:      fcb  'DataMode:',0
nd_msg:            fcb  'NoData',0
sds_msg:           fcb  'To PC',0
d2m_msg:           fcb  'To Mem',0

