;**************************************************************************
;**********************    AA908 DIAGNOSTICS      *************************
;**************************************************************************

Diags:  jsr     LCD_Clear
        ldhx    #BannerMsg
        jsr     disp_msg                ;display "Diagnostics" on 1st line
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    #SelectMsg
        jsr     disp_msg                ;"Select Command"
        brclr   ConfigPB,portG,*        ;await release of CONFIG PB

        ldhx    #DiagCmndTable          ;set up pointer to DiagCmndTable
        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #DiagMsgTable           ;set up pointer to DiagMsgTable
        sthx    MsgTabStart
        sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jsr     get_dial_selection
        bra     Diags

exitdiag:
        brclr   ENC_PB,portf,*         ;wait here until Encoder PB released
        jmp     Startmain

BannerMsg:      fcb     '  Diagnostics',0
SelectMsg:      fcb     'Select Function',0

;*******************************
;   DIAG JUMP TABLE

DiagCmndTable:
        fdb     DDS
        fdb     Button
        fdb     Shaft_Encoder
        fdb     Raw_Enc
        fdb     Piezo_Beep
        fdb     Timing_Test
        fdb     Read_EEPROM
        fdb     Erase_EEPROM
        fdb     Write_EEPROM
        fdb     Display_Test
        fdb     Display_ADC
        fdb     Exit_Diags
        fdb     DebugMonitor

;********************************
;        Diag MESSAGE TABLE

DiagMsgTable:
        fcb     'Set DDS',0
        fcb     'Pushbuttons',0
        fcb     'Rotary Encoder',0
        fcb     'Raw Encoder',0
        fcb     'Piezo-Beep',0
        fcb     'Timing Test',0
        fcb     'Read EEPROM',0
        fcb     'Erase EEPROM',0
        fcb     'Write EEPROM',0
        fcb     'Display Test',0
        fcb     'Display ADC',0
        fcb     'Exit Diags',0
        fcb     'Debug Monitor',0
        fcb     0                       ;end of list character


$page
;*********************************
;        COMMAND PROCESSORS

;------------------------
; DDS Test
;------------------------
DDS:    jsr     LCD_clear               ;clear the LCD
        jsr     Wait64ms
        ldhx    #DDS_Msg
        jsr     Disp_Msg                ;display line 1 test message
        bsr     SetDDSto1MHz
        bsr     SetDDSto1MHz
        bsr     SetDDSto1MHz
        brset   ConfigPB,portG,*        ;wait here until Config PB pressed
        brclr   ConfigPB,portG,*        ;then released
        rts

DDS_Msg:
        fcb     'DDS = 1 MHz',0

             ;Set DDS = 1 MHz
SetDDSto1Mhz:
        mov     #$29,DDS_w4             ;DDS programming register W4 (freq-b7  ...  freq-b0)
        mov     #$5C,DDS_w3             ;DDS programming register W3 (freq-b15 ...  freq-b8)
        mov     #$8F,DDS_w2             ;DDS programming register W2 (freq-b23 ...  freq-b16)
        mov     #$02,DDS_w1             ;DDS programming register W1 (freq-b31 ...  freq-b24)
        mov     #$00,DDS_w0             ;DDS programming register W0 (phase-b4 ...  control-0)
        jsr     set_DDS                 ;set the DDS
        rts

;------------------------------
; Display ADC Readings
;------------------------------
Display_ADC:
        jsr     SetDDSto1MHz           ;set DDS to 1 MHz for these measurmements
        jsr     SetDDSto1MHz
        jsr     SetDDSto1MHz
        jsr     SetDDSto1MHz

        jsr     LCD_Clear
        ldhx    #ADC_Names
        jsr     Disp_Msg

da_1:   lda     #$C0                    ;point to spot on 2nd line
        jsr     Cmnd2LCD
        lda     #CHAN_Vf
        jsr     read_ADC                ;read Vf
        jsr     disp_ADC

        lda     #$C4                    ;point to spot on 2nd line
        jsr     Cmnd2LCD
        lda     #CHAN_Vr
        jsr     read_ADC                ;read Vr
        jsr     disp_ADC

        lda     #$C8                    ;point to spot on 2nd line
        jsr     Cmnd2LCD
        lda     #CHAN_Vz                ;read Vz
        jsr     read_ADC
        jsr     disp_ADC

        lda     #$CC                    ;point to spot on 2nd line
        jsr     Cmnd2LCD
        lda     #CHAN_Va                ;read Va
        jsr     read_ADC
        jsr     disp_ADC

        brclr   Enc_PB,portF,da_2       ;ENC PB pressed?
        bra     da_1                    ;no, continue looping

             ;exit
da_2:   brclr   Enc_PB,portF,*          ;await release of ENC PB
        rts


            ;display the ADC reading in A as two ascii characters
disp_ADC:
        jsr     _dblchar                ;convert reg A to two equiv ascii chars in HX
        tha
        jsr     data2LCD
        txa
        jsr     data2LCD
        rts

ADC_Names:   fcb  'VF  VR  VZ  VA  ',0

;------------------------------
; Button Test
;------------------------------
Button:
        jsr     LCD_Clear
        jsr     Wait64ms
        jsr     ClearLCDLine2
        jsr     Wait64ms
        lda     #$C0                     ;point to 2nd line on LCD
        jsr     Cmnd2LCD
        ldhx    #Buttons
        jsr     Disp_Msg

bt1:    lda     #$CA                     ;point to first button position on 2nd line
        jsr     Cmnd2LCD

        brclr   ConfigPB,portg,bt2
        bsr     Display0
        bra     bt3
bt2:    bsr     Display1

bt3:    brclr   ModePB,porth,bt4
        bsr     Display0
        bra     bt5
bt4:    bsr     Display1

bt5:    brclr   BandPB,porth,bt6
        bsr     Display0
        bra     bt7
bt6:    bsr     Display1

bt7:    brclr   ScanPB,portg,bt8
        bsr     Display0
        bra     bt11
bt8:    bsr     Display1

              ;exit when both SCAN and BAND are pressed
bt11:   brset   ConfigPB,portG,bt1
        brset   ScanPB,portG,bt1
        brclr   ScanPB,portG,*            ;await release of Scan
        brclr   ConfigPB,portG,*          ;await release of Config
        rts

Display1:
        lda     #'1'
        jsr     data2LCD
        rts

Display0:
        lda     #'0'
        jsr     data2LCD
        rts

Buttons:
        fcb     'Buttons = ',0

;------------------------------
; Shaft Encoder Test
;------------------------------
Shaft_Encoder:
        jsr     LCD_Clear
        jsr     ClearLCDLine2
        lda     #$C0                     ;point to 2nd line on LCD
        jsr     Cmnd2LCD
        ldhx    #EncoderMsg
        jsr     Disp_Msg
        mov     #$30,enc_tens            ;init the turns counter
        mov     #$30,enc_units
se1:    lda     #4                       ;dial sensitivity
        jsr     Read_Encoder             ;process the encoder turns
        bcs     se3                      ;exit if ENC PB pressed
        beq     se2
        jsr     increase_digit_display   ;CW turn detected
        bra     se1
se2:    jsr     decrease_digit_display   ;CCW turn detected
        bra     se1
se3:    brclr   ENC_PB,portf,*           ;wait here until Encoder PB released
        rts                              ;then return

EncoderMsg:
        fcb     'Encoder = 00',0

             ;encoder turned down (ccw)

Decrease_Digit_Display:
        dec     enc_units             ;decrement units digit
        lda     #$2F                  ;units at min?
        cmp     enc_units
        beq     dwn1a                 ;yes, roll down
        bra     dwn1b                 ;no,just continue

dwn1a:  mov     #$39,enc_units        ;roll down units to 9
        dec     enc_tens              ;and decr tens
        lda     #$2F
        cmp     enc_tens              ;tens at min?
        beq     dwn1c                 ;yes, reset cntr to 00
        bra     dwn1b                 ;no, just continue

dwn1c:  lda     #$39
        sta     enc_units             ;reset tens to zero
        sta     enc_tens              ; and units to zero

dwn1b:  jsr     update_digits         ;update counter digits
        rts

             ;encoder turned up (cw)

Increase_Digit_Display:
        inc     enc_units             ;increment units digit
        lda     #$3A                  ;units at max?
        cmp     enc_units
        beq     up1a                  ;yes, roll over
        bra     up1b                  ;no, just continue

up1a:   mov     #$30,enc_units        ;roll over units to zero
        inc     enc_tens              ;and bump tens
        lda     #$3A                  ;tens at max?
        cmp     enc_tens
        beq     up1c                  ;yes, reset cntr to 00
        bra     up1b                  ;no, just continue

up1c:   lda     #$30
        sta     enc_units             ;reset tens to zero
        sta     enc_tens              ; and units to zero

up1b:   jsr     update_digits         ;update counter digits
        rts

update_digits:
        lda     #$CA                  ;point to tens digit on 2nd line
        jsr     Cmnd2LCD
        lda     enc_tens              ;send the tens digit there
        jsr     Data2LCD
        lda     #$CB                  ;point to units digit on 2nd line
        jsr     Cmnd2LCD
        lda     enc_units             ;send the units digit there
        jsr     Data2LCD
        rts

;------------------------------
; Raw Encoder Test
;------------------------------

Raw_Enc:
        jsr     LCD_Clear
        jsr     ClearLCDLine2
        lda     #$C0                    ;point to 2nd line on LCD
        jsr     Cmnd2LCD
        ldhx    #RawEncMsg
        jsr     Disp_Msg

raw0:   lda     #$C2
        jsr     Cmnd2LCD
        brset   5,portf,raw1            ;test ENC A
        ldhx    #LoMsg                  ;A=LO
        bra     raw2
raw1:   ldhx    #HiMsg
raw2:   jsr     Disp_msg
        lda     #$C8
        jsr     Cmnd2LCD
        brset   4,portf,raw3
        ldhx    #LoMsg
        bra     raw4
raw3:   ldhx    #HiMsg
raw4:   jsr     Disp_Msg

             ;exit if ENC PB pressed
        brset   ENC_PB,portf,raw0       ;loop if ENC PB not pressed
        brclr   ENC_PB,portf,*          ;waiut until ENC PB is released
        rts

RawEncMsg:   fcb    'A=    B=  ',0
HiMsg:       fcb    'HI',0
LoMsg:       fcb    'LO',0

;------------------------------
; Piezo_LED Beep Test
;------------------------------

Piezo_Beep:
        jsr     LCD_clear
        ldhx    #Piezo_Msg2           ;"Piezo Test"
        jsr     disp_msg

        mov     #65,Tone
pb1:    Tone_On                       ;(macro definition in macro.asm)
        bclr    BusyLED,portc         ;turn Busy LED on
        jsr    Wait500ms
        brclr   Enc_PB,portF,pb2      ;exit if Enc PB pressed
        Tone_Off
        bset    BusyLED,portc         ;turn Busy LED off
        jsr     Wait500ms
        brset   Enc_PB,portF,pb1      ;do again if Enc PB not pressed
        rts                           ;and return to main loop

pb2:    brclr   Enc_PB,portF,*        ;wait for Enc PB release, then exit

        Tone_Off
        bset    Keyline,portD         ;LED off
        rts

Piezo_Msg2:
        fcb     'Piezo-LED blink',0

;------------------------------
; Timing Test
;------------------------------
Timing_Test:
        sei
tt1:    bset    HeartbeatLED,portF
;        jsr     Wait100us             ;change this Wait to reflect desired test
        bclr    HeartbeatLED,portF     ;else, without the waits, it's a cycle timing test
;        jsr     Wait100us             ;change this Wait to reflect desired test
        bra     tt1
        cli
        rts

Timing_Msg:
        fcb     'Cycle Timing',0

;------------------------------
; Read EEPROM Test
;------------------------------
Read_EEPROM:
        jsr     LCD_clear             ;clear the LCD
        jsr     Wait64ms
        ldhx    #Read_Msg
        jsr     Disp_Msg              ;display line 1 test message
        lda     #$C0
        jsr     Cmnd2LCD              ;point to second line

        ldhx    #$800                 ;EEPROM mem is 800-9FF
        mov     #16,EEbyte_count      ;display 16 bytes/line
ree2:   lda     ,x                    ;get the EEPROM byte
        jsr     data2LCD              ;display EEPROM data to second line
        aix     #1
        dbnz    EEbyte_count,ree2     ;not done with 16 chars yet?
        brset   ConfigPB,portG,*      ;press SCAN PB to continue
        brclr   ConfigPB,portG,*      ;and await release of Config PB
        rts                           ;then return to Diag menus

Read_Msg:
        fcb     'EEPROM Read',0

;------------------------------
; Erase EEPROM Test
;------------------------------
Erase_EEPROM:
        jsr     LCD_clear             ;clear the LCD
        jsr     Wait64ms
        ldhx    #Erase_Msg
        jsr     Disp_Msg              ;display line 1 test message
        lda     #$C0
        jsr     Cmnd2LCD              ;point to second line
        jsr     Erase_EEPROM_All      ;clear all 512 locations
        ldhx    #$800                 ;EEPROM mem is 800-9FF
        mov     #16,EEbyte_count      ;display 16 bytes/line
eee2:   lda     ,x                    ;get the EEPROM byte
        jsr     data2LCD              ;display EEPROM data to second line
        aix     #1
        dbnz    EEbyte_count,eee2     ;not done with 16 chars yet?
        brset   ConfigPB,portG,*      ;press SCAN PB to continue
        brclr   ConfigPB,portG,*      ;and await release of Config PB
        rts                           ;then return to Diag menus

Erase_Msg:
        fcb     'EEPROM erased!',0

;------------------------------
; Write EEPROM Test
;------------------------------
Write_EEPROM:
        jsr     LCD_clear              ;clear the LCD
        jsr     Wait64ms
        ldhx    #Wr_Msg
        jsr     Disp_Msg               ;display line 1 test message
        lda     #$C0
        jsr     Cmnd2LCD              ;point to second line
        ldhx    #Wr_Data
        jsr     Disp_Msg

             ;transfer from input buffer to EEPROM

        ldhx    #$800                 ;start of EEPROM = 800
        sthx    to_addr
        ldhx    #Wr_Data           ;point to start of 16 bytes to be written
        sthx    from_addr
we1:    ldhx    from_addr
        lda     ,x                    ;get char from input buffer
        beq     we_exit               ;exit if EOT terminator found (0)
        aix     #1
        sthx    from_addr
        ldhx    to_addr
        jsr     Write_EEPROM_byte     ;write reg A to loc (hx) in EEPROM area
        aix     #1
        sthx    to_addr
        bra     we1

we_exit:
        brset   ConfigPB,portG,*      ;press SCAN PB to continue
        brclr   ConfigPB,portG,*      ;and await release of Config PB
        rts                           ;then return to Diag menus

Wr_Msg:
        fcb     'EEPROM Write',0
Wr_Data:
        fcb     'PalStar AntAnal',0

;------------------------------
; Display Test
;------------------------------
Display_Test:
        jsr     LCD_clear              ;clear the LCD
        jsr     Wait64ms
        lda     #$80                   ;point to first char, line 1
        jsr     Cmnd2LCD
        ldhx    #Line1Msg
        jsr     Disp_Msg               ;display line 1 test message
        lda     #$C0                   ;point to first char, line 2
        jsr     Cmnd2LCD
        ldhx    #Line2Msg
        jsr     Disp_Msg
        brset   ConfigPB,portG,*      ;press SCAN PB to continue
        brclr   ConfigPB,portG,*      ;and await release of Config PB
        rts                           ;then return to Diag menus

Line1Msg:
        fcb     '  Display Test  ',0
Line2Msg:
        fcb     '1234567890abcdef',0

;------------------------------
; Debug Monitor
;------------------------------
DebugMonitor:
        swi                           ; Software Interrupt enters HCmon
        rts

;------------------------
; Exit Diags
;------------------------
Exit_Diags:
        pulh                          ; pop return address off stack
        pulh
        brclr   ENC_PB,portf,*         ;wait here until Encoder PB released
        jmp     StartMain             ; and go back to start mainline again

