;*********************************************************************************
;*********************************************************************************
;                           SCAN Process
;*********************************************************************************
;*********************************************************************************

Scan_process:
        brclr   ScanPB,portG,*          ;await button release
        tone_off                        ;turn tone off while in Scan menu
        clr     DDS_active              ;clear flag to prevent display of SWR
        jsr     wait100ms
        jsr     LCD_Clear
        ldhx    #Scan1_msg
        jsr     disp_msg                ;display "Scanning" on first line
        lda     #$C0
        jsr     Cmnd2LCD                ;point to 2nd line next
        ldhx    #Scan2_msg              ;display "(please wait")
        jsr     disp_msg

        mov     #$00,F_min+0            ;init default min freq registers to 1 MHz
        mov     #$01,F_min+1
        mov     #$00,F_min+2
        mov     #$00,F_min+3
        mov     #$00,F_min+4
        mov     #$00,F_min+5
        mov     #$00,F_min+6
        mov     #9,WW_min               ;init the SWR min stores
        mov     #9,DD_min

        mov     Scan_Start+0,LCD_dat+0  ;set up the Start Scan frequency
        mov     Scan_Start+1,LCD_dat+1
        mov     Scan_Start+2,LCD_dat+2
        mov     Scan_Start+3,LCD_dat+3
        mov     Scan_Start+4,LCD_dat+4
        mov     Scan_Start+5,LCD_dat+5
        mov     Scan_Start+6,LCD_dat+6

        mov     #$70,ADCLK              ;select clock divide of 011 and set ADICLK bit

        bclr    BusyLED,portc           ;turn on Busy LED

        jsr	Send_Scan		;send the Scan setup to host

scan1:  ldhx    StepPtr                 ;point to desired BCD to increment
        jsr     Bcd_Inc                 ;increment that digit
        jsr     Mult                    ;compute the DDS word
        jsr     Set_DDS                 ;send the control word to the DDS
        jsr     Calc_SWR                ;Calculate SWR
        jsr     Check4Min               ;see if current reading is a minimum
        jsr	Send_SWR		;send SWR to host
        brclr   ScanPB,portG,exit_scan  ;if ScanPB is pressed, then early exit

            ;End of scan range?

        lda     LCD_dat+0
        sub     Scan_End+0              ;10 MHz digits match yet?
        bne     scan1                   ;no

        lda     LCD_dat+1               ;yes
        sub     Scan_End+1              ;1 MHz digits match yet?
        bne     scan1                   ;no

        lda     LCD_dat+2               ;yes
        sub     Scan_End+2              ;100 KHz digits match yet?
        bne     scan1                   ;no
        lda     #2
        cmp     StepPtr+1               ;are we incrementing 100 KHz digit?
        beq     exit_scan               ;yes, that's as far as we check for match

        lda     LCD_dat+3
        sub     Scan_End+3              ;10 KHz digits match yet?
        bne     scan1                   ;no
        lda     #3
        cmp     StepPtr+1               ;are we incrementing 10 KHz digit?
        beq     exit_scan               ;yes, that's as far as we check for match

        lda     LCD_dat+4
        sub     Scan_End+4              ;1 kHz digits match yet?
        bne     scan1                   ;no
        lda     #4
        cmp     StepPtr+1               ;are we incrementing 1 KHz digit?
        beq     exit_scan               ;yes, that's as far as we check for match

        lda     LCD_dat+5
        sub     Scan_End+5              ;100 Hz digits match yet?
        bne     scan1                   ;no
        lda     #5
        cmp     StepPtr+1               ;are we incrementing 100 Hz digit
        beq     exit_scan               ;yes, that's as far as we check for match

        lda     LCD_dat+6
        sub     Scan_End+6              ;10 Hz digits match yet?
        bne     scan1                   ;no
        lda     #6
        cmp     StepPtr+1               ;are we incrementing 10 Hz digit
        beq     exit_scan               ;yes, that's as far as we check for match

        swi                             ;error if get here

exit_scan:

	jsr	Send_EoS		;end of scan to host

        bset    BusyLED,portc           ;turn off Busy LED
        brclr   ScanPB,portG,*          ;wait here until ScanPB is released

        jmp     Impedance               ;go directly into Impedance Mode with freq at min SWR point

Scan1_msg:  fcb  '~~~ SCANNING ~~~',0
Scan2_msg:  fcb  '  (please wait)',0

;-------------------------------------------------
; Send the SCAN Freqs to Link908 program running on PC

Send_Scan:
        lda     DataMode
        cmp     #Data2Mem               ;Save data from Scan to SEEPROM?
        beq     send_freq_mem           ;yes, go process
        cmp     #SendDuringScan         ;send the data during the scan?
        beq     send_freq_pc            ;yes, go process
        rts                             ;else just exit

            ; Send freq limits to SEEPROM
send_freq_mem:
        rts                             ;INSERT SEEPROM USAGE HERE

            ; Send the freq limits now to the PC
send_freq_pc:
	jsr	Send_SoS		;start of scan to host

	ldhx	#Scan_Start		;send the Start Freq
	lda	#6
	bsr	Send_nHex
	jsr	_crlf

	ldhx	#Scan_End		;send the End Freq
	lda	#6
	bsr	Send_nHex
	jsr	_crlf

	ldhx	#StepPtr		;send the Step Index
	lda	#2
	bsr	Send_nHex
	jsr	_crlf

	rts

;-------------------------------------------------
; Send 'n' hex bytes as ascii - enter with 'n' in A

Send_nHex:
        psha                            ;save 'n'
        lda     DataMode
        cmp     #Data2Mem               ;Save data from Scan to SEEPROM?
        beq     send_n_mem              ;yes, go process
        cmp     #SendDuringScan         ;send the data during the scan?
        beq     send_n                  ;yes, do it
        pula
        rts                             ;else just exit

             ; Send data to SEEPROM
send_n_mem:
	pula				;clean up the stack
        rts                             ;INSERT SEEPROM USAGE HERE

             ; Send data to PC during Scan
send_n: pula                            ;get 'n' back
        psha				;and save it again
send_nlp:
	lda	0,x			;get the byte
	jsr	Send_LSN		;convert and send it
	aix	#1			;move to the next location
	dbnz	1,sp,send_nlp		;loop until done
	pula				;clean up the stack
	rts

;-------------------------------------------------
; Send SWR as ascii WW.DD

Send_SWR:
        lda     DataMode
        cmp     #Data2Mem               ;Save data from Scan to SEEPROM?
        beq     s_swr_mem               ;yes, go process
        cmp     #SendDuringScan         ;send the data during the scan?
        beq     s_swr_pc                ;yes, do it
        rts                             ;else just exit

            ; Send SWR to SEEPROM
s_swr_mem:
        rts                             ;INSERT SEEPROM USAGE HERE

            ; Send SWR to PC
s_swr_pc:
        ldhx    SWRx100+2               ;get SWRx100(2:3) into intacc1(2:3)
        sthx    intacc1+2
        jsr     BIN2BCD                 ;convert intacc1(2:3) to BCD in bcd10000_bcd_1000_bcd100_bcd10_bcd1
	lda	bcd100			;get integer
	jsr	Send_LSN		;convert and send it

	lda	#'.'
	jsr	_putch			;send the decimal point

	lda	bcd10			;get the decimal value
	jsr	Send_LSN		;convert and send it

	lda	bcd1
	jsr	Send_LSN		;convert and send it

             ; Send R to pc
	lda	#','			;send a comma
	jsr	_putch

        jsr     Calc_Z			;calculate R
        jsr     Calc_R
        ldhx    R+2                     ;get R(2:3) into intacc1(2:3)
        sthx    intacc1+2
        jsr     Bin2BCD                 ;convert intacc1(2:3) to BCD in bcd10000_bcd_1000_bcd100_bcd10_bcd1

        ldhx	#bcd1000
        lda	#4
        jsr	Send_nHex

             ; Send X to pc
	lda	#','			;send a comma
	jsr	_putch

        jsr     Calc_X
        ldhx    Reactance               ;get 2-byte reactance value (computed in Calc_X) into intacc1(2:3)
        sthx    intacc1+2
        jsr     Bin2BCD                 ;convert intacc1(2:3) to BCD in bcd10000_bcd_1000_bcd100_bcd10_bcd1

        ldhx	#bcd1000
        lda	#3
        jsr	Send_nHex

	lda	#'.'
	jsr	_putch			;send the decimal point

	lda	bcd1			;get the decimal value
	jsr	Send_LSN		;convert and send it


	jsr	_crlf

	rts

;-------------------------------------------------
; Send START of scan

Send_SoS:
        lda     DataMode
        cmp     #Data2Mem               ;Save data from Scan to SEEPROM?
        beq     s_sos_mem               ;yes, go process
        cmp     #SendDuringScan         ;send the data during the scan?
        beq     s_sos_pc                ;yes, do it
        rts                             ;else just exit

            ; Send start flag to SEEPROM
s_sos_mem:
        rts                             ;INSERT SEEPROM USAGE HERE

            ; Send start flag to PC during Scan
s_sos_pc:
	lda	#'S'			;send terminator
	jsr	_putch
	jsr	_crlf
	rts

;-------------------------------------------------
; Send END of scan

Send_EoS:
        lda     DataMode
        cmp     #Data2Mem               ;Save data from Scan to SEEPROM?
        beq     send_e_mem              ;yes, go process
        cmp     #SendDuringScan         ;send the data during the scan?
        beq     send_e_pc               ;yes, do it
        rts                             ;else just exit

            ; Send end flag to SEEPROM
send_e_mem:
        rts                             ;INSERT SEEPROM USAGE HERE

            ; Send end flag to PC during scan
send_e_pc:
	lda	#'E'			;send terminator
	jsr	_putch
	jsr	_crlf
	rts

;-------------------------------------------------
; Send LSNibble as ASCII

Send_LSN:
	and	#$0f			;remove MSNibble
	add	#'0'			;make it ascii
	cmp	#'9'			;see if it's alpha
	bls	no_alpha

	add	#7			;make it alpha

no_alpha:
        psha                            ;save ascii-ized nibble
        lda     DataMode                ;check for where to send the data
        cmp     #Data2Mem               ;Save data from Scan to SEEPROM?
        beq     send_lsn_mem            ;yes, go process
        cmp     #SendDuringScan         ;send the data during the scan?
        beq     send_lsn_pc             ;yes, send it to PC
        pula                            ;clean up stack
        rts                             ;else just exit

            ; Send LSNibble to SEEPROM
send_lsn_mem:
        pula                            ;retrieve the data byte
        rts                             ;INSERT SEEPROM USAGE HERE

            ; Send LSNibble to PC during Scan
send_lsn_pc:
        pula                            ;retrieve the data byte
	jmp	_putch			;send it to host

;-------------------------------------------------
; Disp_F - Display frequency from F_min array

Disp_F:
        lda     #$86                    ;point to frequency point on line 1
        jsr     Cmnd2LCD
        lda     F_min+0                 ;display 10M digit
        ora     #$30
        jsr     Data2LCD
        lda     F_min+1                 ;display 1M digit
        ora     #$30
        jsr     Data2LCD
        lda     #','                    ;display comma
        jsr     Data2LCD
        lda     F_min+2                 ;display 100K digit
        ora     #$30
        jsr     Data2LCD
        lda     F_min+3                 ;display 10K digit
        ora     #$30
        jsr     Data2LCD
        lda     F_min+4                 ;display 1K digit
        ora     #$30
        jsr     Data2LCD
        lda     #'.'                    ;display decimal point
        jsr     Data2LCD
        lda     F_min+5                 ;display 100 Hz digit
        ora     #$30
        jsr     Data2LCD
        lda     F_min+6                 ;display 10 Hz digit
        ora     #$30
        jsr     Data2LCD
        rts

;**************************************************************************
; Check4Min - Check for minimum SWR during scan

Check4Min:
        lda     WW                      ;compare current SWR whole number (WW)
        cmp     WW_min                  ;to the min WW
        bne     c4m_1                   ;current whole numbers are not same
        lda     DD                      ;WW is same, so check DD numbers
        cmp     DD_min
        bcs     c4m_3                   ;current DD < min, so currrent is new min!
        rts                             ;current DD > min, so just exit

c4m_1:  bcs     c4m_3                   ;current WW < min, so current is new min!
        rts                             ;current WW > min, so just exit

            ;New min found!  Save current in min locations
c4m_3:  mov     WW,WW_min
        mov     DD,DD_min
        mov     LCD_Dat+0,F_min+0
        mov     LCD_Dat+1,F_min+1
        mov     LCD_Dat+2,F_min+2
        mov     LCD_Dat+3,F_min+3
        mov     LCD_Dat+4,F_min+4
        mov     LCD_Dat+5,F_min+5
        mov     LCD_Dat+6,F_min+6
        rts
